@@ -14,7 +14,7 @@
 //               mpirun -np 4 ex26p -d ceed-cuda
 //
 // Description:  This example code demonstrates the use of MFEM to define a
-//               simple finite element discretization of the Laplace problem
+//               simple finite element discretization of the Poisson problem
 //               -Delta u = 1 with homogeneous Dirichlet boundary conditions
 //               as in Example 1.
 //
@@ -58,7 +58,7 @@ public:
       }
    }
 
-   virtual ~DiffusionMultigrid()
+   ~DiffusionMultigrid() override
    {
       delete amg;
    }
