@@ -318,7 +318,7 @@ int main(int argc, char *argv[])
    // 10. Set up the parallel linear form b(.) which corresponds to the
    //     right-hand side of the FEM linear system.
    ParComplexLinearForm b(&fespace, conv);
-   b.Vector::operator=(0.0);
+   b = 0.0;
 
    // 11a. Define the solution vector u as a parallel complex finite element
    //      grid function corresponding to fespace. Initialize u to equal zero.
