@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -61,10 +61,10 @@ public:
    void SetProtocol(const std::string &protocol);
 
    /// Save the collection and a FMS blueprint root file
-   virtual void Save();
+   void Save() override;
 
    /// Load the collection based blueprint data
-   virtual void Load(int cycle = 0);
+   void Load(int cycle = 0) override;
 };
 
 } // namespace mfem
