@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -9,6 +9,8 @@
 // terms of the BSD-3 license. We welcome feedback and contributions, see file
 // CONTRIBUTING.md for details.
 
+#ifdef MFEM_USE_MOONOLITH
+
 #include "transfer.hpp"
 
 #include "par_moonolith_instance.hpp"
@@ -30,3 +32,5 @@ void InitTransfer(int argc, char *argv[], MPI_Comm comm)
 #endif
 
 } // namespace mfem
+
+#endif // MFEM_USE_MOONOLITH
