@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -75,15 +75,22 @@ MFEM_REGISTER_TMOP_KERNELS(real_t, MinDetJpr_Kernel_3D,
 }
 
 real_t TMOPNewtonSolver::MinDetJpr_3D(const FiniteElementSpace *fes,
-                                      const Vector &X) const
+                                      const Vector &D) const
 {
    const ElementDofOrdering ordering = ElementDofOrdering::LEXICOGRAPHIC;
-   const Operator *R = fes->GetElementRestriction(ordering);
-   Vector XE(R->Height(), Device::GetDeviceMemoryType());
-   XE.UseDevice(true);
-   R->Mult(X, XE);
 
-   const DofToQuad &maps = fes->GetFE(0)->GetDofToQuad(ir, DofToQuad::TENSOR);
+   const Operator *RD = fes->GetElementRestriction(ordering);
+   Vector DE(RD->Height(), Device::GetDeviceMemoryType());
+   DE.UseDevice(true);
+   RD->Mult(D, DE);
+
+   const Operator *RX = x_0.FESpace()->GetElementRestriction(ordering);
+   Vector XE(RX->Height(), Device::GetDeviceMemoryType());
+   XE.UseDevice(true);
+   RX->Mult(x_0, XE);
+   XE += DE;
+
+   auto maps = fes->GetTypicalFE()->GetDofToQuad(ir, DofToQuad::TENSOR);
    const int NE = fes->GetMesh()->GetNE();
    const int NQ = ir.GetNPoints();
    const int D1D = maps.ndof;
