@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -122,13 +122,11 @@ TargetConstructor::ComputeAllElementTargets<3>(const FiniteElementSpace &fes,
    MFEM_ASSERT(target_type == IDEAL_SHAPE_UNIT_SIZE || nodes != nullptr, "");
    const Mesh *mesh = fes.GetMesh();
    const int NE = mesh->GetNE();
-   // Quick return for empty processors:
-   if (NE == 0) { return true; }
    const int dim = mesh->Dimension();
    MFEM_VERIFY(mesh->GetNumGeometries(dim) <= 1,
                "mixed meshes are not supported");
    MFEM_VERIFY(!fes.IsVariableOrder(), "variable orders are not supported");
-   const FiniteElement &fe = *fes.GetFE(0);
+   const FiniteElement &fe = *fes.GetTypicalFE();
    MFEM_VERIFY(fe.GetGeomType() == Geometry::CUBE, "");
    const DenseMatrix &W = Geometries.GetGeomToPerfGeomJac(Geometry::CUBE);
    const DofToQuad::Mode mode = DofToQuad::TENSOR;
