@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -21,6 +21,7 @@
 #ifdef MFEM_USE_CUDA
 #define MFEM_USE_CUDA_OR_HIP
 #define MFEM_DEVICE __device__
+#define MFEM_HOST __host__
 #define MFEM_LAMBDA __host__
 // #define MFEM_HOST_DEVICE __host__ __device__ // defined in config/config.hpp
 #define MFEM_DEVICE_SYNC MFEM_GPU_CHECK(cudaDeviceSynchronize())
@@ -28,16 +29,14 @@
 // Define a CUDA error check macro, MFEM_GPU_CHECK(x), where x returns/is of
 // type 'cudaError_t'. This macro evaluates 'x' and raises an error if the
 // result is not cudaSuccess.
-#define MFEM_GPU_CHECK(x) \
-   do \
-   { \
-      cudaError_t err = (x); \
-      if (err != cudaSuccess) \
-      { \
-         mfem_cuda_error(err, #x, _MFEM_FUNC_NAME, __FILE__, __LINE__); \
-      } \
-   } \
-   while (0)
+#define MFEM_GPU_CHECK(x)                                                      \
+  do {                                                                         \
+    cudaError_t mfem_err_internal_var_name = (x);                              \
+    if (mfem_err_internal_var_name != cudaSuccess) {                           \
+      ::mfem::mfem_cuda_error(mfem_err_internal_var_name, #x, _MFEM_FUNC_NAME, \
+                              __FILE__, __LINE__);                             \
+    }                                                                          \
+  } while (0)
 #endif // MFEM_USE_CUDA
 
 // Define the MFEM inner threading macros
