@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -72,9 +72,9 @@ bool Device::mem_types_set = false;
 
 Device::Device()
 {
-   if (getenv("MFEM_MEMORY") && !mem_host_env && !mem_device_env)
+   if (GetEnv("MFEM_MEMORY") && !mem_host_env && !mem_device_env)
    {
-      std::string mem_backend(getenv("MFEM_MEMORY"));
+      std::string mem_backend(GetEnv("MFEM_MEMORY"));
       if (mem_backend == "host")
       {
          mem_host_env = true;
@@ -139,9 +139,9 @@ Device::Device()
       mm.Configure(host_mem_type, device_mem_type);
    }
 
-   if (getenv("MFEM_DEVICE"))
+   if (GetEnv("MFEM_DEVICE"))
    {
-      std::string device(getenv("MFEM_DEVICE"));
+      std::string device(GetEnv("MFEM_DEVICE"));
       Configure(device);
       device_env = true;
    }
@@ -150,6 +150,9 @@ Device::Device()
 
 Device::~Device()
 {
+#ifdef MFEM_USE_MPI
+   Hypre::Finalize();
+#endif
    if ( device_env && !destroy_mm) { return; }
    if (!device_env &&  destroy_mm && !mem_host_env)
    {
@@ -255,7 +258,15 @@ void Device::Configure(const std::string &device, const int device_id)
    destroy_mm = true;
 
 #ifdef MFEM_USE_MPI
-   Hypre::InitDevice();
+#if defined(HYPRE_USING_GPU) && (MFEM_HYPRE_VERSION >= 23100)
+   // Skip the call to Hypre::InitDevice() if HYPRE is not initialized, e.g.
+   // * if running a serial code
+   // * if running with the environment variable MFEM_DEVICE set.
+   if (HYPRE_Initialized())
+   {
+      Hypre::InitDevice();
+   }
+#endif
 #endif
 }
 
@@ -585,4 +596,157 @@ void Device::Setup(const int device_id)
    if (Allows(Backend::DEBUG_DEVICE)) { ngpu = 1; }
 }
 
+MemoryType Device::QueryMemoryType(void* ptr)
+{
+   // from HYPRE's hypre_GetPointerLocation
+   MemoryType res = MemoryType::HOST;
+#if defined(MFEM_USE_CUDA)
+   struct cudaPointerAttributes attr;
+
+#if (CUDART_VERSION >= 11000)
+   MFEM_GPU_CHECK(cudaPointerGetAttributes(&attr, ptr));
+#else
+   cudaPointerGetAttributes(&attr, ptr);
+   if (err != cudaSuccess)
+   {
+      /* clear the error */
+      cudaGetLastError();
+   }
+#endif
+   switch (attr.type)
+   {
+      case cudaMemoryTypeUnregistered:
+         // host
+         break;
+      case cudaMemoryTypeHost:
+         res = MemoryType::HOST_PINNED;
+         break;
+      case cudaMemoryTypeDevice:
+         res = MemoryType::DEVICE;
+         break;
+      case cudaMemoryTypeManaged:
+         res = MemoryType::MANAGED;
+         break;
+   }
+
+#elif defined(MFEM_USE_HIP)
+   struct hipPointerAttribute_t attr;
+
+   hipError_t err = hipPointerGetAttributes(&attr, ptr);
+   if (err != hipSuccess)
+   {
+      if (err == hipErrorInvalidValue)
+      {
+         // host memory
+         /* clear the error */
+         hipGetLastError();
+      }
+      else
+      {
+         MFEM_GPU_CHECK(err);
+      }
+   }
+   else if (attr.isManaged)
+   {
+      res = MemoryType::MANAGED;
+   }
+#if (HIP_VERSION_MAJOR >= 6)
+   else if (attr.type == hipMemoryTypeDevice)
+#else  // (HIP_VERSION_MAJOR < 6)
+   else if (attr.memoryType == hipMemoryTypeDevice)
+#endif // (HIP_VERSION_MAJOR >= 6)
+   {
+      res = MemoryType::DEVICE;
+   }
+#if (HIP_VERSION_MAJOR >= 6)
+   else if (attr.type == hipMemoryTypeHost)
+#else  // (HIP_VERSION_MAJOR < 6)
+   else if (attr.memoryType == hipMemoryTypeHost)
+#endif // (HIP_VERSION_MAJOR >= 6)
+   {
+      res = MemoryType::HOST_PINNED;
+   }
+#if (HIP_VERSION_MAJOR >= 6)
+   else if (attr.type == hipMemoryTypeUnregistered)
+   {
+      // host memory
+   }
+#endif
+#endif
+   return res;
+}
+
+void Device::DeviceMem(size_t *free, size_t *total)
+{
+#if defined(MFEM_USE_CUDA)
+   cudaMemGetInfo(free, total);
+#elif defined(MFEM_USE_HIP)
+   hipMemGetInfo(free, total);
+#else
+   // not compiled with GPU support
+   if (free)
+   {
+      *free = 0;
+   }
+   if (*total)
+   {
+      *total = 0;
+   }
+#endif
+}
+
+int Device::NumMultiprocessors(int dev)
+{
+#if defined(MFEM_USE_CUDA)
+   int res;
+   cudaDeviceGetAttribute(&res, cudaDevAttrMultiProcessorCount, dev);
+   return res;
+#elif defined(MFEM_USE_HIP)
+   int res;
+   hipDeviceGetAttribute(&res, hipDeviceAttributeMultiprocessorCount, dev);
+   return res;
+#else
+   // not compiled with GPU support
+   return 0;
+#endif
+}
+
+int Device::NumMultiprocessors()
+{
+   int dev = 0;
+#if defined(MFEM_USE_CUDA)
+   cudaGetDevice(&dev);
+#elif defined(MFEM_USE_HIP)
+   hipGetDevice(&dev);
+#endif
+   return NumMultiprocessors(dev);
+}
+
+int Device::WarpSize(int dev)
+{
+#if defined(MFEM_USE_CUDA)
+   int res;
+   cudaDeviceGetAttribute(&res, cudaDevAttrWarpSize, dev);
+   return res;
+#elif defined(MFEM_USE_HIP)
+   int res;
+   hipDeviceGetAttribute(&res, hipDeviceAttributeWarpSize, dev);
+   return res;
+#else
+   // not compiled with GPU support
+   return 0;
+#endif
+}
+
+int Device::WarpSize()
+{
+   int dev = 0;
+#if defined(MFEM_USE_CUDA)
+   cudaGetDevice(&dev);
+#elif defined(MFEM_USE_HIP)
+   hipGetDevice(&dev);
+#endif
+   return WarpSize(dev);
+}
+
 } // mfem
