@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -101,6 +101,9 @@ public:
    /// For backward compatibility define Size to be synonym of Width()
    int Size() const { return Width(); }
 
+   // Total size = width*height
+   int TotalSize() const { return width*height; }
+
    /// Change the size of the DenseMatrix to s x s.
    void SetSize(int s) { SetSize(s, s); }
 
@@ -133,10 +136,10 @@ public:
    real_t Trace() const;
 
    /// Returns reference to a_{ij}.
-   virtual real_t &Elem(int i, int j);
+   real_t &Elem(int i, int j) override;
 
    /// Returns constant reference to a_{ij}.
-   virtual const real_t &Elem(int i, int j) const;
+   const real_t &Elem(int i, int j) const override;
 
    /// Matrix vector multiplication.
    void Mult(const real_t *x, real_t *y) const;
@@ -148,7 +151,7 @@ public:
    void Mult(const Vector &x, real_t *y) const;
 
    /// Matrix vector multiplication.
-   virtual void Mult(const Vector &x, Vector &y) const;
+   void Mult(const Vector &x, Vector &y) const override;
 
    /// Multiply a vector with the transpose matrix.
    void MultTranspose(const real_t *x, real_t *y) const;
@@ -160,17 +163,17 @@ public:
    void MultTranspose(const Vector &x, real_t *y) const;
 
    /// Multiply a vector with the transpose matrix.
-   virtual void MultTranspose(const Vector &x, Vector &y) const;
+   void MultTranspose(const Vector &x, Vector &y) const override;
 
    using Operator::Mult;
    using Operator::MultTranspose;
 
    /// y += a * A.x
-   virtual void AddMult(const Vector &x, Vector &y, const real_t a = 1.0) const;
+   void AddMult(const Vector &x, Vector &y, const real_t a = 1.0) const override;
 
    /// y += a * A^t x
-   virtual void AddMultTranspose(const Vector &x, Vector &y,
-                                 const real_t a = 1.0) const;
+   void AddMultTranspose(const Vector &x, Vector &y,
+                         const real_t a = 1.0) const override;
 
    /// y += a * A.x
    void AddMult_a(real_t a, const Vector &x, Vector &y) const;
@@ -199,7 +202,7 @@ public:
    { return InnerProduct(x.GetData(), y.GetData()); }
 
    /// Returns a pointer to the inverse matrix.
-   virtual MatrixInverse *Inverse() const;
+   MatrixInverse *Inverse() const override;
 
    /// Replaces the current matrix with its inverse
    void Invert();
@@ -207,6 +210,10 @@ public:
    /// Replaces the current matrix with its square root inverse
    void SquareRootInverse();
 
+   /// Replaces the current matrix with its exponential
+   /// (currently only supports 2x2 matrices)
+   void Exponential();
+
    /// Calculates the determinant of the matrix
    /// (optimized for 2x2, 3x3, and 4x4 matrices)
    real_t Det() const;
@@ -452,8 +459,9 @@ public:
    int CheckFinite() const { return mfem::CheckFinite(HostRead(), height*width); }
 
    /// Prints matrix to stream out.
-   virtual void Print(std::ostream &out = mfem::out, int width_ = 4) const;
-   virtual void PrintMatlab(std::ostream &out = mfem::out) const;
+   void Print(std::ostream &out = mfem::out, int width_ = 4) const override;
+   void PrintMatlab(std::ostream &out = mfem::out) const override;
+   virtual void PrintMathematica(std::ostream &out = mfem::out) const;
    /// Prints the transpose matrix to stream out.
    virtual void PrintT(std::ostream &out = mfem::out, int width_ = 4) const;
 
@@ -688,11 +696,11 @@ public:
     *
     * @return status set to true if successful, otherwise, false.
     */
-   virtual bool Factor(int m, real_t TOL = 0.0);
+   bool Factor(int m, real_t TOL = 0.0) override;
 
    /** Assuming L.U = P.A factored data of size (m x m), compute |A|
        from the diagonal values of U and the permutation information. */
-   virtual real_t Det(int m) const;
+   real_t Det(int m) const override;
 
    /** Assuming L.U = P.A factored data of size (m x m), compute X <- A X,
        for a matrix X of size (m x n). */
@@ -708,14 +716,14 @@ public:
 
    /** Assuming L.U = P.A factored data of size (m x m), compute X <- A^{-1} X,
        for a matrix X of size (m x n). */
-   virtual void Solve(int m, int n, real_t *X) const;
+   void Solve(int m, int n, real_t *X) const override;
 
    /** Assuming L.U = P.A factored data of size (m x m), compute X <- X A^{-1},
        for a matrix X of size (n x m). */
    void RightSolve(int m, int n, real_t *X) const;
 
    /// Assuming L.U = P.A factored data of size (m x m), compute X <- A^{-1}.
-   virtual void GetInverseMatrix(int m, real_t *X) const;
+   void GetInverseMatrix(int m, real_t *X) const override;
 
    /** Given an (n x m) matrix A21, compute X2 <- X2 - A21 X1, for matrices X1,
        and X2 of size (m x r) and (n x r), respectively. */
@@ -787,11 +795,11 @@ public:
     *
     * @return status set to true if successful, otherwise, false.
     */
-   virtual bool Factor(int m, real_t TOL = 0.0);
+   bool Factor(int m, real_t TOL = 0.0) override;
 
    /** Assuming LL^t = A factored data of size (m x m), compute |A|
        from the diagonal values of L */
-   virtual real_t Det(int m) const;
+   real_t Det(int m) const override;
 
    /** Assuming L.L^t = A factored data of size (m x m), compute X <- L X,
        for a matrix X of size (m x n). */
@@ -811,14 +819,14 @@ public:
 
    /** Assuming L.L^t = A factored data of size (m x m), compute X <- A^{-1} X,
        for a matrix X of size (m x n). */
-   virtual void Solve(int m, int n, real_t *X) const;
+   void Solve(int m, int n, real_t *X) const override;
 
    /** Assuming L.L^t = A factored data of size (m x m), compute X <- X A^{-1},
        for a matrix X of size (n x m). */
    void RightSolve(int m, int n, real_t *X) const;
 
    /// Assuming L.L^t = A factored data of size (m x m), compute X <- A^{-1}.
-   virtual void GetInverseMatrix(int m, real_t *X) const;
+   void GetInverseMatrix(int m, real_t *X) const override;
 
 };
 
@@ -855,13 +863,13 @@ public:
    /// Factor a new DenseMatrix of the same size
    void Factor(const DenseMatrix &mat);
 
-   virtual void SetOperator(const Operator &op);
+   void SetOperator(const Operator &op) override;
 
    /// Matrix vector multiplication with the inverse of dense matrix.
    void Mult(const real_t *x, real_t *y) const;
 
    /// Matrix vector multiplication with the inverse of dense matrix.
-   virtual void Mult(const Vector &x, Vector &y) const;
+   void Mult(const Vector &x, Vector &y) const override;
 
    /// Multiply the inverse matrix by another matrix: X = A^{-1} B.
    void Mult(const DenseMatrix &B, DenseMatrix &X) const;
@@ -1169,6 +1177,31 @@ public:
       tdata.Wrap(ext_data, i*j*k, false);
    }
 
+   /// @brief Reset the DenseTensor to use the given external Memory @a mem and
+   /// dimensions @a i, @a j, and @a k.
+   ///
+   /// If @a own_mem is false, the DenseTensor will not own any of the pointers
+   /// of @a mem.
+   ///
+   /// Note that when @a own_mem is true, the @a mem object can be destroyed
+   /// immediately by the caller but `mem.Delete()` should NOT be called since
+   /// the DenseTensor object takes ownership of all pointers owned by @a mem.
+   void NewMemoryAndSize(const Memory<real_t> &mem, int i, int j, int k,
+                         bool own_mem)
+   {
+      tdata.Delete();
+      Mk.UseExternalData(NULL, i, j);
+      nk = k;
+      if (own_mem)
+      {
+         tdata = mem;
+      }
+      else
+      {
+         tdata.MakeAlias(mem, 0, i*j*k);
+      }
+   }
+
    /// Sets the tensor elements equal to constant c
    DenseTensor &operator=(real_t c);
 
@@ -1267,7 +1300,8 @@ public:
    ~DenseTensor() { tdata.Delete(); }
 };
 
-/** @brief Compute the LU factorization of a batch of matrices
+/** @brief Compute the LU factorization of a batch of matrices. Calls
+    BatchedLinAlg::LUFactor.
 
     Factorize n matrices of size (m x m) stored in a dense tensor overwriting it
     with the LU factors. The factorization is such that L.U = Piv.A, where A is
@@ -1278,7 +1312,7 @@ public:
     @param [in] TOL optional fuzzy comparison tolerance. Defaults to 0.0. */
 void BatchLUFactor(DenseTensor &Mlu, Array<int> &P, const real_t TOL = 0.0);
 
-/** @brief Solve batch linear systems
+/** @brief Solve batch linear systems. Calls BatchedLinAlg::LUSolve.
 
     Assuming L.U = P.A for n factored matrices (m x m), compute x <- A x, for n
     companion vectors.
@@ -1289,7 +1323,6 @@ void BatchLUFactor(DenseTensor &Mlu, Array<int> &P, const real_t TOL = 0.0);
     dimension m x n. */
 void BatchLUSolve(const DenseTensor &Mlu, const Array<int> &P, Vector &X);
 
-
 // Inline methods
 
 inline real_t &DenseMatrix::operator()(int i, int j)
