@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -12,13 +12,99 @@
 #ifndef MFEM_ODE
 #define MFEM_ODE
 
+#include "../general/communication.hpp"
 #include "../config/config.hpp"
 #include "operator.hpp"
-#include "../general/communication.hpp"
+#include <vector>
+#include <memory>
 
 namespace mfem
 {
 
+/// An interface for storing state of previous timesteps
+class ODEStateData
+{
+public:
+   /// Get the maximum number of stored stages
+   virtual int MaxSize() const = 0;
+
+   /// Get the current number of stored stages
+   virtual int Size() const = 0;
+
+   /// Get the ith state vector
+   virtual const Vector &Get(int i) const = 0;
+
+   /// Get the ith state vector - non-const version
+   virtual Vector &Get(int i) = 0;
+
+   /// Get the ith state vector - with a copy
+   virtual void Get(int i, Vector &vec) const = 0;
+
+   /// Set the ith state vector
+   virtual void Set(int i, Vector &state) = 0;
+
+   /// Add state vector and increment state size
+   virtual void Append(Vector &state) = 0;
+
+   /// Virtual destructor
+   virtual ~ODEStateData() = default;
+};
+
+/// An implementation of ODEStateData that stores states in an std::vector<Vector>
+class ODEStateDataVector : public ODEStateData
+{
+private:
+   MemoryType mem_type;
+   int ss, smax;
+   std::vector<Vector> data;
+   Array<int> idx;
+
+public:
+   ODEStateDataVector (int smax): smax(smax)
+   {
+      data.resize(smax);
+      idx.SetSize(smax);
+      ss = 0;
+   };
+
+   /// Set the number of stages and the size of the vectors
+   void  SetSize(int vsize, MemoryType mem_type);
+
+   /// Shift the stage counter for the next timestep
+   inline void ShiftStages()
+   {
+      for (int i = 0; i < smax; i++) { idx[i] = (++idx[i])%smax; }
+   };
+
+   /// Increment the stage counter
+   void Increment() { ss++; ss = std::min(ss,smax); };
+
+   /// Reset the stage counter
+   void Reset() { ss = 0; };
+
+   /// Reference access to the ith vector.
+   inline Vector & operator[](int i) { return data[idx[i]]; };
+
+   /// Const reference access to the ith vector.
+   inline const Vector &operator[](int i) const { return data[idx[i]]; };
+
+   /// Print state data
+   void Print(std::ostream &os = mfem::out) const ;
+
+   int  MaxSize() const override { return smax; };
+
+   int  Size() const override { return ss; };
+
+   const Vector &Get(int i) const override;
+   Vector &Get(int i) override;
+   void Get(int i, Vector &vec) const override;
+
+   void Set(int i, Vector &state) override;
+
+   void Append(Vector &state) override;
+};
+
+
 /// Abstract class for solving systems of ODEs: dx/dt = f(x,t)
 class ODESolver
 {
@@ -92,26 +178,48 @@ public:
       while (t < tf) { Step(x, t, dt); }
    }
 
-   /// Function for getting and setting the state vectors
-   virtual int   GetMaxStateSize() { return 0; }
-   virtual int   GetStateSize() { return 0; }
-   virtual const Vector &GetStateVector(int i)
-   {
-      mfem_error("ODESolver has no state vectors");
-      Vector *s = NULL; return *s; // Make some compiler happy
-   }
-   virtual void  GetStateVector(int i, Vector &state)
-   {
-      mfem_error("ODESolver has no state vectors");
-   }
-   virtual void  SetStateVector(int i, Vector &state)
-   {
-      mfem_error("ODESolver has no state vectors");
-   }
+   /// Returns how many State vectors the ODE requires
+   virtual int GetStateSize() { return 0; };
+
+   // Help info for ODESolver options
+   static MFEM_EXPORT std::string ExplicitTypes;
+   static MFEM_EXPORT std::string ImplicitTypes;
+   static MFEM_EXPORT std::string Types;
+
+   /// Function for selecting the desired ODESolver (Explicit and Implicit)
+   /// Returns an ODESolver pointer based on an type
+   /// Caller gets ownership of the object and is responsible for its deletion
+   static MFEM_EXPORT std::unique_ptr<ODESolver> Select(const int ode_solver_type);
+
+   /// Function for selecting the desired Explicit ODESolver
+   /// Returns an ODESolver pointer based on an type
+   /// Caller gets ownership of the object and is responsible for its deletion
+   static MFEM_EXPORT std::unique_ptr<ODESolver> SelectExplicit(
+      const int ode_solver_type);
+
+   /// Function for selecting the desired Implicit ODESolver
+   /// Returns an ODESolver pointer based on an type
+   /// Caller gets ownership of the object and is responsible for its deletion
+   static MFEM_EXPORT std::unique_ptr<ODESolver> SelectImplicit(
+      const int ode_solver_type);
 
    virtual ~ODESolver() { }
 };
 
+/// Abstract class for an ODESolver that has state history implemented as ODEStateData
+class ODESolverWithStates : public ODESolver
+{
+public:
+   /// Returns the StateData
+   virtual ODEStateData& GetState() = 0;
+
+   /// Returns the StateData
+   virtual const ODEStateData& GetState() const = 0;
+
+   /// Returns how many State vectors the ODE requires
+   virtual int GetStateSize() { return GetState().MaxSize(); };
+};
+
 
 /// The classical forward Euler method
 class ForwardEulerSolver : public ODESolver
@@ -217,196 +325,13 @@ public:
 class RK8Solver : public ExplicitRKSolver
 {
 private:
-   static const real_t a[66], b[12], c[11];
+   static MFEM_EXPORT const real_t a[66], b[12], c[11];
 
 public:
    RK8Solver() : ExplicitRKSolver(12, a, b, c) { }
 };
 
 
-/** An explicit Adams-Bashforth method. */
-class AdamsBashforthSolver : public ODESolver
-{
-private:
-   int s, smax;
-   const real_t *a;
-   Vector *k;
-   Array<int> idx;
-   ODESolver *RKsolver;
-   real_t dt_;
-
-   inline bool print()
-   {
-#ifdef MFEM_USE_MPI
-      return Mpi::IsInitialized() ? Mpi::Root() : true;
-#else
-      return true;
-#endif
-   }
-
-public:
-   AdamsBashforthSolver(int s_, const real_t *a_);
-
-   void Init(TimeDependentOperator &f_) override;
-
-   void Step(Vector &x, real_t &t, real_t &dt) override;
-
-   int  GetMaxStateSize() override { return smax; };
-   int  GetStateSize() override { return s; };
-   const Vector &GetStateVector(int i) override;
-   void GetStateVector(int i, Vector &state) override;
-   void SetStateVector(int i, Vector &state) override;
-
-   ~AdamsBashforthSolver()
-   {
-      if (RKsolver) { delete RKsolver; }
-      delete [] k;
-   }
-};
-
-/** A 1-stage, 1st order AB method.  */
-class AB1Solver : public AdamsBashforthSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[1];
-
-public:
-   AB1Solver() : AdamsBashforthSolver(1, a) { }
-};
-
-/** A 2-stage, 2nd order AB method.  */
-class AB2Solver : public AdamsBashforthSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[2];
-
-public:
-   AB2Solver() : AdamsBashforthSolver(2, a) { }
-};
-
-/** A 3-stage, 3rd order AB method.  */
-class AB3Solver : public AdamsBashforthSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[3];
-
-public:
-   AB3Solver() : AdamsBashforthSolver(3, a) { }
-};
-
-/** A 4-stage, 4th order AB method.  */
-class AB4Solver : public AdamsBashforthSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[4];
-
-public:
-   AB4Solver() : AdamsBashforthSolver(4, a) { }
-};
-
-/** A 5-stage, 5th order AB method.  */
-class AB5Solver : public AdamsBashforthSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[5];
-
-public:
-   AB5Solver() : AdamsBashforthSolver(5, a) { }
-};
-
-
-/** An implicit Adams-Moulton method. */
-class AdamsMoultonSolver : public ODESolver
-{
-private:
-   int s, smax;
-   const real_t *a;
-   Vector *k;
-   Array<int> idx;
-   ODESolver *RKsolver;
-   real_t dt_;
-
-   inline bool print()
-   {
-#ifdef MFEM_USE_MPI
-      return Mpi::IsInitialized() ? Mpi::Root() : true;
-#else
-      return true;
-#endif
-   }
-
-public:
-   AdamsMoultonSolver(int s_, const real_t *a_);
-
-   void Init(TimeDependentOperator &f_) override;
-
-   void Step(Vector &x, real_t &t, real_t &dt) override;
-
-   int  GetMaxStateSize() override { return smax-1; };
-   int  GetStateSize() override { return s-1; };
-   const Vector &GetStateVector(int i) override;
-   void GetStateVector(int i, Vector &state) override;
-   void SetStateVector(int i, Vector &state) override;
-
-   ~AdamsMoultonSolver()
-   {
-      if (RKsolver) { delete RKsolver; }
-      delete [] k;
-   };
-};
-
-/** A 0-stage, 1st order AM method. */
-class AM0Solver : public AdamsMoultonSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[1];
-
-public:
-   AM0Solver() : AdamsMoultonSolver(0, a) { }
-};
-
-
-/** A 1-stage, 2nd order AM method. */
-class AM1Solver : public AdamsMoultonSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[2];
-
-public:
-   AM1Solver() : AdamsMoultonSolver(1, a) { }
-};
-
-/** A 2-stage, 3rd order AM method. */
-class AM2Solver : public AdamsMoultonSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[3];
-
-public:
-   AM2Solver() : AdamsMoultonSolver(2, a) { }
-};
-
-/** A 3-stage, 4th order AM method. */
-class AM3Solver : public AdamsMoultonSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[4];
-
-public:
-   AM3Solver() : AdamsMoultonSolver(3, a) { }
-};
-
-/** A 4-stage, 5th order AM method. */
-class AM4Solver : public AdamsMoultonSolver
-{
-private:
-   static MFEM_EXPORT const real_t a[5];
-
-public:
-   AM4Solver() : AdamsMoultonSolver(4, a) { }
-};
-
-
 /// Backward Euler ODE solver. L-stable.
 class BackwardEulerSolver : public ODESolver
 {
@@ -490,9 +415,9 @@ protected:
    Vector k, y;
 
 public:
-   virtual void Init(TimeDependentOperator &f_);
+   void Init(TimeDependentOperator &f_) override;
 
-   virtual void Step(Vector &x, real_t &t, real_t &dt);
+   void Step(Vector &x, real_t &t, real_t &dt) override;
 };
 
 
@@ -504,9 +429,9 @@ protected:
    Vector k, y, z;
 
 public:
-   virtual void Init(TimeDependentOperator &f_);
+   void Init(TimeDependentOperator &f_) override;
 
-   virtual void Step(Vector &x, real_t &t, real_t &dt);
+   void Step(Vector &x, real_t &t, real_t &dt) override;
 };
 
 
@@ -518,40 +443,192 @@ protected:
    Vector k, y, z;
 
 public:
-   virtual void Init(TimeDependentOperator &f_);
+   void Init(TimeDependentOperator &f_) override;
 
-   virtual void Step(Vector &x, real_t &t, real_t &dt);
+   void Step(Vector &x, real_t &t, real_t &dt) override;
 };
 
 
 /// Generalized-alpha ODE solver from "A generalized-α method for integrating
 /// the filtered Navier-Stokes equations with a stabilized finite element
 /// method" by K.E. Jansen, C.H. Whiting and G.M. Hulbert.
-class GeneralizedAlphaSolver : public ODESolver
+class GeneralizedAlphaSolver : public ODESolverWithStates
 {
+   ODEStateDataVector state;
+
 protected:
-   mutable Vector xdot,k,y;
+
+   mutable Vector k,y;
    real_t alpha_f, alpha_m, gamma;
-   int  nstate;
 
    void SetRhoInf(real_t rho_inf);
-   void PrintProperties(std::ostream &out = mfem::out);
+   void PrintProperties(std::ostream &os = mfem::out);
 public:
 
-   GeneralizedAlphaSolver(real_t rho = 1.0) { SetRhoInf(rho); };
-
+   GeneralizedAlphaSolver(real_t rho = 1.0) : state(1) { SetRhoInf(rho); };
    void Init(TimeDependentOperator &f_) override;
-
    void Step(Vector &x, real_t &t, real_t &dt) override;
 
-   int  GetMaxStateSize() override { return 1; };
-   int  GetStateSize() override { return nstate; };
-   const Vector &GetStateVector(int i) override;
-   void GetStateVector(int i, Vector &state) override;
-   void SetStateVector(int i, Vector &state) override;
+   ODEStateData& GetState() override { return state; }
+   const ODEStateData& GetState() const override { return state; }
 };
 
 
+/** An explicit Adams-Bashforth method. */
+class AdamsBashforthSolver : public ODESolverWithStates
+{
+private:
+   const real_t *a;
+   const int stages;
+   real_t dt_;
+   ODEStateDataVector state;
+
+protected:
+   std::unique_ptr<ODESolver> RKsolver;
+
+   inline bool print()
+   {
+#ifdef MFEM_USE_MPI
+      return Mpi::IsInitialized() ? Mpi::Root() : true;
+#else
+      return true;
+#endif
+   }
+
+   void CheckTimestep(real_t dt);
+
+public:
+   AdamsBashforthSolver(int s_, const real_t *a_);
+   void Init(TimeDependentOperator &f_) override;
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+
+   ODEStateData& GetState() override { return state; }
+   const ODEStateData& GetState() const override { return state; }
+};
+
+/** A 1-stage, 1st order AB method.  */
+class AB1Solver : public AdamsBashforthSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[1];
+
+public:
+   AB1Solver() : AdamsBashforthSolver(1, a) { }
+};
+
+/** A 2-stage, 2nd order AB method.  */
+class AB2Solver : public AdamsBashforthSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[2];
+
+public:
+   AB2Solver() : AdamsBashforthSolver(2, a) { RKsolver.reset(new RK2Solver()); }
+};
+
+/** A 3-stage, 3rd order AB method.  */
+class AB3Solver : public AdamsBashforthSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[3];
+
+public:
+   AB3Solver() : AdamsBashforthSolver(3, a) { RKsolver.reset(new RK3SSPSolver()); }
+};
+
+/** A 4-stage, 4th order AB method.  */
+class AB4Solver : public AdamsBashforthSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[4];
+
+public:
+   AB4Solver() : AdamsBashforthSolver(4, a) { RKsolver.reset(new RK4Solver()); }
+};
+
+/** A 5-stage, 5th order AB method.  */
+class AB5Solver : public AdamsBashforthSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[5];
+
+public:
+   AB5Solver() : AdamsBashforthSolver(5, a) { RKsolver.reset(new RK6Solver()); }
+};
+
+
+/** An implicit Adams-Moulton method. */
+class AdamsMoultonSolver : public ODESolverWithStates
+{
+private:
+   const real_t *a;
+   const int stages;
+   real_t dt_;
+   ODEStateDataVector state;
+
+protected:
+   std::unique_ptr<ODESolver> RKsolver;
+
+   inline bool print()
+   {
+#ifdef MFEM_USE_MPI
+      return Mpi::IsInitialized() ? Mpi::Root() : true;
+#else
+      return true;
+#endif
+   }
+
+   void CheckTimestep(real_t dt);
+
+public:
+   AdamsMoultonSolver(int s_, const real_t *a_);
+   void Init(TimeDependentOperator &f_) override;
+   void Step(Vector &x, real_t &t, real_t &dt) override;
+
+   ODEStateData& GetState() override { return state; }
+   const ODEStateData& GetState() const override { return state; }
+};
+
+/** A 1-stage, 2nd order AM method. */
+class AM1Solver : public AdamsMoultonSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[2];
+
+public:
+   AM1Solver() : AdamsMoultonSolver(1, a) { RKsolver.reset(new SDIRK23Solver()); }
+};
+
+/** A 2-stage, 3rd order AM method. */
+class AM2Solver : public AdamsMoultonSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[3];
+
+public:
+   AM2Solver() : AdamsMoultonSolver(2, a) { RKsolver.reset(new SDIRK23Solver()); }
+};
+
+/** A 3-stage, 4th order AM method. */
+class AM3Solver : public AdamsMoultonSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[4];
+
+public:
+   AM3Solver() : AdamsMoultonSolver(3, a) { RKsolver.reset(new SDIRK23Solver()); }
+};
+
+/** A 4-stage, 5th order AM method. */
+class AM4Solver : public AdamsMoultonSolver
+{
+private:
+   static MFEM_EXPORT const real_t a[5];
+
+public:
+   AM4Solver() : AdamsMoultonSolver(4, a) { RKsolver.reset(new SDIRK34Solver()); }
+};
+
 /// The SIASolver class is based on the Symplectic Integration Algorithm
 /// described in "A Symplectic Integration Algorithm for Separable Hamiltonian
 /// Functions" by J. Candy and W. Rozmus, Journal of Computational Physics,
@@ -630,9 +707,10 @@ protected:
    /// Pointer to the associated TimeDependentOperator.
    SecondOrderTimeDependentOperator *f;  // f(.,.,t) : R^n x R^n --> R^n
    MemoryType mem_type;
+   ODEStateDataVector state;
 
 public:
-   SecondOrderODESolver() : f(NULL) { mem_type = MemoryType::HOST; }
+   SecondOrderODESolver() : f(NULL), state(1) { mem_type = MemoryType::HOST; }
 
    /// Associate a TimeDependentOperator with the ODE solver.
    /** This method has to be called:
@@ -680,6 +758,8 @@ public:
          sequence, then the ODE solver must be re-initialized by calling Init()
          between the two Step() calls. */
    virtual void Step(Vector &x, Vector &dxdt, real_t &t, real_t &dt) = 0;
+   void EulerStep(Vector &x, Vector &dxdt, real_t &t, real_t &dt);
+   void MidPointStep(Vector &x, Vector &dxdt, real_t &t, real_t &dt);
 
    /// Perform time integration from time @a t [in] to time @a tf [in].
    /** @param[in,out] x    Approximate solution.
@@ -705,22 +785,18 @@ public:
       while (t < tf) { Step(x, dxdt, t, dt); }
    }
 
-   /// Function for getting and setting the state vectors
-   virtual int   GetMaxStateSize() { return 0; };
-   virtual int   GetStateSize() { return 0; }
-   virtual const Vector &GetStateVector(int i)
-   {
-      mfem_error("ODESolver has no state vectors");
-      Vector *s = NULL; return *s; // Make some compiler happy
-   }
-   virtual void  GetStateVector(int i, Vector &state)
-   {
-      mfem_error("ODESolver has no state vectors");
-   }
-   virtual void  SetStateVector(int i, Vector &state)
-   {
-      mfem_error("ODESolver has no state vectors");
-   }
+   /// Functions for getting the state vectors
+   ODEStateData&  GetState() { return state; }
+   const ODEStateData&  GetState() const { return state; }
+
+   /// Returns how many State vectors the ODE requires
+   int GetStateSize() { return GetState().MaxSize(); };
+
+   /// Help info for SecondOrderODESolver options
+   static MFEM_EXPORT std::string Types;
+
+   /// Function selecting the desired SecondOrderODESolver
+   static MFEM_EXPORT SecondOrderODESolver *Select(const int ode_solver_type);
 
    virtual ~SecondOrderODESolver() { }
 };
@@ -731,17 +807,18 @@ public:
 class NewmarkSolver : public SecondOrderODESolver
 {
 private:
-   Vector d2xdt2;
-
    real_t beta, gamma;
-   bool first;
+   bool no_mult;
 
 public:
-   NewmarkSolver(real_t beta_ = 0.25, real_t gamma_ = 0.5) { beta = beta_; gamma = gamma_; };
+   NewmarkSolver(real_t beta_ = 0.25, real_t gamma_ = 0.5, bool no_mult_ = false)
+   {
+      beta = beta_;
+      gamma = gamma_;
+      no_mult = no_mult_;
+   };
 
-   void PrintProperties(std::ostream &out = mfem::out);
-
-   void Init(SecondOrderTimeDependentOperator &f_) override;
+   void PrintProperties(std::ostream &os = mfem::out);
 
    void Step(Vector &x, Vector &dxdt, real_t &t, real_t &dt) override;
 };
@@ -773,13 +850,14 @@ public:
 class GeneralizedAlpha2Solver : public SecondOrderODESolver
 {
 protected:
-   Vector xa,va,aa,d2xdt2;
+   Vector xa,va,aa;
    real_t alpha_f, alpha_m, beta, gamma;
-   int nstate;
+   bool no_mult;
 
 public:
-   GeneralizedAlpha2Solver(real_t rho_inf = 1.0)
+   GeneralizedAlpha2Solver(real_t rho_inf = 1.0, bool no_mult_ = false)
    {
+      no_mult = no_mult_;
       rho_inf = (rho_inf > 1.0) ? 1.0 : rho_inf;
       rho_inf = (rho_inf < 0.0) ? 0.0 : rho_inf;
 
@@ -789,17 +867,12 @@ public:
       gamma   = 0.5 + alpha_m - alpha_f;
    };
 
-   void PrintProperties(std::ostream &out = mfem::out);
+   void PrintProperties(std::ostream &os = mfem::out);
 
    void Init(SecondOrderTimeDependentOperator &f_) override;
 
    void Step(Vector &x, Vector &dxdt, real_t &t, real_t &dt) override;
 
-   int  GetMaxStateSize() override { return 1; };
-   int  GetStateSize() override { return nstate; };
-   const Vector &GetStateVector(int i) override;
-   void GetStateVector(int i, Vector &state) override;
-   void SetStateVector(int i, Vector &state) override;
 };
 
 /// The classical midpoint method.
