@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -779,7 +779,8 @@ void RectangularConstrainedOperator::MultTranspose(const Vector &x,
 }
 
 real_t PowerMethod::EstimateLargestEigenvalue(Operator& opr, Vector& v0,
-                                              int numSteps, real_t tolerance, int seed)
+                                              int numSteps, real_t tolerance,
+                                              int seed)
 {
    v1.SetSize(v0.Size());
    if (seed != 0)
