@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -28,6 +28,83 @@
 namespace mfem
 {
 
+/**
+ * Reducer for helping to compute L2-norms. Given two partial results:
+ * a0 = sum_i (|v_i|/a1)^2
+ * b0 = sum_j (|v_j|/b1)^2 (j disjoint from i for vector v)
+ * computes:
+ * a1 = max(a1, b1)
+ * a0 = (a1 == 0 ? 0 : sum_{k in union(i,j)} (|v_k|/a1)^2)
+ *
+ * This form is resiliant against overflow/underflow, similar to std::hypot
+ */
+struct L2Reducer
+{
+   using value_type = DevicePair<real_t, real_t>;
+   static MFEM_HOST_DEVICE void Join(value_type& a, const value_type &b)
+   {
+      real_t scale = fmax(a.second, b.second);
+      if (scale > 0)
+      {
+         real_t s = a.second / scale;
+         a.first *= s * s;
+         s = b.second / scale;
+         a.first += b.first * s * s;
+         a.second = scale;
+      }
+   }
+
+   static MFEM_HOST_DEVICE void SetInitialValue(value_type &a)
+   {
+      a.first = 0;
+      a.second = 0;
+   }
+};
+
+/**
+ * Reducer for helping to compute Lp-norms. Given two partial results:
+ * a0 = sum_i (|v_i|/a1)^p
+ * b0 = sum_j (|v_j|/b1)^p (j disjoint from i for vector v)
+ * computes:
+ * a1 = max(a1, b1)
+ * a0 = (a1 == 0 ? 0 : sum_{k in union(i,j)} (|v_k|/a1)^p)
+ *
+ * This form is resiliant against overflow/underflow, similar to std::hypot
+ */
+struct LpReducer
+{
+   real_t p;
+   using value_type = DevicePair<real_t, real_t>;
+   MFEM_HOST_DEVICE void Join(value_type& a, const value_type &b) const
+   {
+      real_t scale = fmax(a.second, b.second);
+      if (scale > 0)
+      {
+         a.first = a.first * pow(a.second / scale, p) +
+                   b.first * pow(b.second / scale, p);
+         a.second = scale;
+      }
+   }
+
+   static MFEM_HOST_DEVICE void SetInitialValue(value_type &a)
+   {
+      a.first = 0;
+      a.second = 0;
+   }
+};
+
+static Array<real_t>& vector_workspace()
+{
+   static Array<real_t> instance;
+   return instance;
+}
+
+static Array<DevicePair<real_t, real_t>> &Lpvector_workspace()
+{
+   static Array<DevicePair<real_t, real_t>> instance;
+   return instance;
+}
+
 Vector::Vector(const Vector &v)
 {
    const int s = v.Size();
@@ -104,6 +181,7 @@ const real_t &Vector::Elem(int i) const
 
 real_t Vector::operator*(const real_t *v) const
 {
+   HostRead();
    real_t dot = 0.0;
 #ifdef MFEM_USE_LEGACY_OPENMP
    #pragma omp parallel for reduction(+:dot)
@@ -142,11 +220,8 @@ Vector &Vector::operator=(const Vector &v)
 
 Vector &Vector::operator=(Vector &&v)
 {
-   data = std::move(v.data);
-   // Self-assignment-safe way to move v.size to size:
-   const auto size_tmp = v.size;
-   v.size = 0;
-   size = size_tmp;
+   v.Swap(*this);
+   if (this != &v) { v.Destroy(); }
    return *this;
 }
 
@@ -805,6 +880,30 @@ void Vector::Print_HYPRE(std::ostream &os) const
    os.flags(old_fmt);
 }
 
+void Vector::PrintMathematica(std::ostream & os) const
+{
+   std::ios::fmtflags old_fmt = os.flags();
+   os.setf(std::ios::scientific);
+   std::streamsize old_prec = os.precision(14);
+
+   os << "(* Read file into Mathematica using: "
+      << "myVec = Get[\"this_file_name\"] *)\n";
+   os << "{\n";
+
+   data.Read(MemoryClass::HOST, size);
+   for (int i = 0; i < size; i++)
+   {
+      os << "Internal`StringToMReal[\"" << ZeroSubnormal(data[i]) << "\"]";
+      if (i < size - 1) { os << ','; }
+      os << '\n';
+   }
+
+   os << "}\n";
+
+   os.precision(old_prec);
+   os.flags(old_fmt);
+}
+
 void Vector::PrintHash(std::ostream &os) const
 {
    os << "size: " << size << '\n';
@@ -834,39 +933,66 @@ real_t Vector::Norml2() const
    // Scale entries of Vector on the fly, using algorithms from
    // std::hypot() and LAPACK's drm2. This scaling ensures that the
    // argument of each call to std::pow is <= 1 to avoid overflow.
-   if (0 == size)
+   if (size == 0)
    {
       return 0.0;
-   } // end if 0 == size
+   }
 
-   data.Read(MemoryClass::HOST, size);
-   if (1 == size)
+   auto m_data = Read(UseDevice());
+   using value_type = DevicePair<real_t, real_t>;
+   value_type res;
+   res.first = 0;
+   res.second = 0;
+   // first compute sum (|m_data|/scale)^2
+   reduce(
+      size, res,
+      [=] MFEM_HOST_DEVICE(int i, value_type &r)
    {
-      return std::abs(data[0]);
-   } // end if 1 == size
-   return kernels::Norml2(size, (const real_t*) data);
+      real_t n = fabs(m_data[i]);
+      if (n > 0)
+      {
+         if (r.second <= n)
+         {
+            real_t arg = r.second / n;
+            r.first = r.first * (arg * arg) + 1;
+            r.second = n;
+         }
+         else
+         {
+            real_t arg = n / r.second;
+            r.first += arg * arg;
+         }
+      }
+   },
+   L2Reducer{}, UseDevice(), Lpvector_workspace());
+   // final answer
+   return res.second * sqrt(res.first);
 }
 
 real_t Vector::Normlinf() const
 {
-   HostRead();
-   real_t max = 0.0;
-   for (int i = 0; i < size; i++)
-   {
-      max = std::max(std::abs(data[i]), max);
-   }
-   return max;
+   if (size == 0) { return 0; }
+
+   auto m_data = Read(UseDevice());
+   real_t res = 0;
+   reduce(
+      size, res,
+   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmax(r, fabs(m_data[i])); },
+   MaxReducer<real_t> {}, UseDevice(), vector_workspace());
+   return res;
 }
 
 real_t Vector::Norml1() const
 {
-   HostRead();
-   real_t sum = 0.0;
-   for (int i = 0; i < size; i++)
-   {
-      sum += std::abs(data[i]);
-   }
-   return sum;
+   if (size == 0) { return 0.0; }
+
+   auto m_data = Read(UseDevice());
+   real_t res = 0;
+   reduce(
+      size, res,
+   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += fabs(m_data[i]); },
+   SumReducer<real_t> {}, UseDevice(), vector_workspace());
+   return res;
 }
 
 real_t Vector::Normlp(real_t p) const
@@ -886,327 +1012,112 @@ real_t Vector::Normlp(real_t p) const
       // Scale entries of Vector on the fly, using algorithms from
       // std::hypot() and LAPACK's drm2. This scaling ensures that the
       // argument of each call to std::pow is <= 1 to avoid overflow.
-      if (0 == size)
+      if (size == 0)
       {
          return 0.0;
-      } // end if 0 == size
-
-      if (1 == size)
-      {
-         return std::abs(data[0]);
-      } // end if 1 == size
-
-      real_t scale = 0.0;
-      real_t sum = 0.0;
-
-      for (int i = 0; i < size; i++)
-      {
-         if (data[i] != 0.0)
-         {
-            const real_t absdata = std::abs(data[i]);
-            if (scale <= absdata)
-            {
-               sum = 1.0 + sum * std::pow(scale / absdata, p);
-               scale = absdata;
-               continue;
-            } // end if scale <= absdata
-            sum += std::pow(absdata / scale, p); // else scale > absdata
-         } // end if data[i] != 0
       }
-      return scale * std::pow(sum, 1.0/p);
+
+      auto m_data = Read(UseDevice());
+      using value_type = DevicePair<real_t, real_t>;
+      value_type res;
+      res.first = 0;
+      res.second = 0;
+      // first compute sum (|m_data|/scale)^p
+      reduce(
+         size, res,
+         [=] MFEM_HOST_DEVICE(int i, value_type &r)
+      {
+         real_t n = fabs(m_data[i]);
+         if (n > 0)
+         {
+            if (r.second <= n)
+            {
+               real_t arg = r.second / n;
+               r.first = r.first * pow(arg, p) + 1;
+               r.second = n;
+            }
+            else
+            {
+               real_t arg = n / r.second;
+               r.first += pow(arg, p);
+            }
+         }
+      },
+      LpReducer{p}, UseDevice(), Lpvector_workspace());
+      // final answer
+      return res.second * pow(res.first, 1.0 / p);
    } // end if p < infinity()
 
    return Normlinf(); // else p >= infinity()
 }
 
-real_t Vector::Max() const
-{
-   if (size == 0) { return -infinity(); }
-
-   HostRead();
-   real_t max = data[0];
-
-   for (int i = 1; i < size; i++)
-   {
-      if (data[i] > max)
-      {
-         max = data[i];
-      }
-   }
-
-   return max;
-}
-
-#ifdef MFEM_USE_CUDA
-static __global__ void cuKernelMin(const int N, real_t *gdsr, const real_t *x)
-{
-   __shared__ real_t s_min[MFEM_CUDA_BLOCKS];
-   const int n = blockDim.x*blockIdx.x + threadIdx.x;
-   if (n>=N) { return; }
-   const int bid = blockIdx.x;
-   const int tid = threadIdx.x;
-   const int bbd = bid*blockDim.x;
-   const int rid = bbd+tid;
-   s_min[tid] = x[n];
-   for (int workers=blockDim.x>>1; workers>0; workers>>=1)
-   {
-      __syncthreads();
-      if (tid >= workers) { continue; }
-      if (rid >= N) { continue; }
-      const int dualTid = tid + workers;
-      if (dualTid >= N) { continue; }
-      const int rdd = bbd+dualTid;
-      if (rdd >= N) { continue; }
-      if (dualTid >= blockDim.x) { continue; }
-      s_min[tid] = fmin(s_min[tid], s_min[dualTid]);
-   }
-   if (tid==0) { gdsr[bid] = s_min[0]; }
-}
-
-static Array<real_t> cuda_reduce_buf;
-
-static real_t cuVectorMin(const int N, const real_t *X)
-{
-   const int tpb = MFEM_CUDA_BLOCKS;
-   const int blockSize = MFEM_CUDA_BLOCKS;
-   const int gridSize = (N+blockSize-1)/blockSize;
-   const int min_sz = (N%tpb)==0? (N/tpb) : (1+N/tpb);
-   cuda_reduce_buf.SetSize(min_sz);
-   Memory<real_t> &buf = cuda_reduce_buf.GetMemory();
-   real_t *d_min = buf.Write(MemoryClass::DEVICE, min_sz);
-   cuKernelMin<<<gridSize,blockSize>>>(N, d_min, X);
-   MFEM_GPU_CHECK(cudaGetLastError());
-   const real_t *h_min = buf.Read(MemoryClass::HOST, min_sz);
-   real_t min = std::numeric_limits<real_t>::infinity();
-   for (int i = 0; i < min_sz; i++) { min = std::min(min, h_min[i]); }
-   return min;
-}
-
-static __global__ void cuKernelDot(const int N, real_t *gdsr,
-                                   const real_t *x, const real_t *y)
-{
-   __shared__ real_t s_dot[MFEM_CUDA_BLOCKS];
-   const int n = blockDim.x*blockIdx.x + threadIdx.x;
-   if (n>=N) { return; }
-   const int bid = blockIdx.x;
-   const int tid = threadIdx.x;
-   const int bbd = bid*blockDim.x;
-   const int rid = bbd+tid;
-   s_dot[tid] = y ? (x[n] * y[n]) : x[n];
-   for (int workers=blockDim.x>>1; workers>0; workers>>=1)
-   {
-      __syncthreads();
-      if (tid >= workers) { continue; }
-      if (rid >= N) { continue; }
-      const int dualTid = tid + workers;
-      if (dualTid >= N) { continue; }
-      const int rdd = bbd+dualTid;
-      if (rdd >= N) { continue; }
-      if (dualTid >= blockDim.x) { continue; }
-      s_dot[tid] += s_dot[dualTid];
-   }
-   if (tid==0) { gdsr[bid] = s_dot[0]; }
-}
-
-static real_t cuVectorDot(const int N, const real_t *X, const real_t *Y)
-{
-   const int tpb = MFEM_CUDA_BLOCKS;
-   const int blockSize = MFEM_CUDA_BLOCKS;
-   const int gridSize = (N+blockSize-1)/blockSize;
-   const int dot_sz = (N%tpb)==0? (N/tpb) : (1+N/tpb);
-   cuda_reduce_buf.SetSize(dot_sz, Device::GetDeviceMemoryType());
-   Memory<real_t> &buf = cuda_reduce_buf.GetMemory();
-   real_t *d_dot = buf.Write(MemoryClass::DEVICE, dot_sz);
-   cuKernelDot<<<gridSize,blockSize>>>(N, d_dot, X, Y);
-   MFEM_GPU_CHECK(cudaGetLastError());
-   const real_t *h_dot = buf.Read(MemoryClass::HOST, dot_sz);
-   real_t dot = 0.0;
-   for (int i = 0; i < dot_sz; i++) { dot += h_dot[i]; }
-   return dot;
-}
-#endif // MFEM_USE_CUDA
-
-#ifdef MFEM_USE_HIP
-static __global__ void hipKernelMin(const int N, real_t *gdsr, const real_t *x)
-{
-   __shared__ real_t s_min[MFEM_HIP_BLOCKS];
-   const int n = hipBlockDim_x*hipBlockIdx_x + hipThreadIdx_x;
-   if (n>=N) { return; }
-   const int bid = hipBlockIdx_x;
-   const int tid = hipThreadIdx_x;
-   const int bbd = bid*hipBlockDim_x;
-   const int rid = bbd+tid;
-   s_min[tid] = x[n];
-   for (int workers=hipBlockDim_x>>1; workers>0; workers>>=1)
-   {
-      __syncthreads();
-      if (tid >= workers) { continue; }
-      if (rid >= N) { continue; }
-      const int dualTid = tid + workers;
-      if (dualTid >= N) { continue; }
-      const int rdd = bbd+dualTid;
-      if (rdd >= N) { continue; }
-      if (dualTid >= hipBlockDim_x) { continue; }
-      s_min[tid] = std::min(s_min[tid], s_min[dualTid]);
-   }
-   if (tid==0) { gdsr[bid] = s_min[0]; }
-}
-
-static Array<real_t> hip_reduce_buf;
-
-static real_t hipVectorMin(const int N, const real_t *X)
-{
-   const int tpb = MFEM_HIP_BLOCKS;
-   const int blockSize = MFEM_HIP_BLOCKS;
-   const int gridSize = (N+blockSize-1)/blockSize;
-   const int min_sz = (N%tpb)==0 ? (N/tpb) : (1+N/tpb);
-   hip_reduce_buf.SetSize(min_sz);
-   Memory<real_t> &buf = hip_reduce_buf.GetMemory();
-   real_t *d_min = buf.Write(MemoryClass::DEVICE, min_sz);
-   hipLaunchKernelGGL(hipKernelMin,gridSize,blockSize,0,0,N,d_min,X);
-   MFEM_GPU_CHECK(hipGetLastError());
-   const real_t *h_min = buf.Read(MemoryClass::HOST, min_sz);
-   real_t min = std::numeric_limits<real_t>::infinity();
-   for (int i = 0; i < min_sz; i++) { min = std::min(min, h_min[i]); }
-   return min;
-}
-
-static __global__ void hipKernelDot(const int N, real_t *gdsr,
-                                    const real_t *x, const real_t *y)
-{
-   __shared__ real_t s_dot[MFEM_HIP_BLOCKS];
-   const int n = hipBlockDim_x*hipBlockIdx_x + hipThreadIdx_x;
-   if (n>=N) { return; }
-   const int bid = hipBlockIdx_x;
-   const int tid = hipThreadIdx_x;
-   const int bbd = bid*hipBlockDim_x;
-   const int rid = bbd+tid;
-   s_dot[tid] = y ? (x[n] * y[n]) : x[n];
-   for (int workers=hipBlockDim_x>>1; workers>0; workers>>=1)
-   {
-      __syncthreads();
-      if (tid >= workers) { continue; }
-      if (rid >= N) { continue; }
-      const int dualTid = tid + workers;
-      if (dualTid >= N) { continue; }
-      const int rdd = bbd+dualTid;
-      if (rdd >= N) { continue; }
-      if (dualTid >= hipBlockDim_x) { continue; }
-      s_dot[tid] += s_dot[dualTid];
-   }
-   if (tid==0) { gdsr[bid] = s_dot[0]; }
-}
-
-static real_t hipVectorDot(const int N, const real_t *X, const real_t *Y)
-{
-   const int tpb = MFEM_HIP_BLOCKS;
-   const int blockSize = MFEM_HIP_BLOCKS;
-   const int gridSize = (N+blockSize-1)/blockSize;
-   const int dot_sz = (N%tpb)==0 ? (N/tpb) : (1+N/tpb);
-   hip_reduce_buf.SetSize(dot_sz);
-   Memory<real_t> &buf = hip_reduce_buf.GetMemory();
-   real_t *d_dot = buf.Write(MemoryClass::DEVICE, dot_sz);
-   hipLaunchKernelGGL(hipKernelDot,gridSize,blockSize,0,0,N,d_dot,X,Y);
-   MFEM_GPU_CHECK(hipGetLastError());
-   const real_t *h_dot = buf.Read(MemoryClass::HOST, dot_sz);
-   real_t dot = 0.0;
-   for (int i = 0; i < dot_sz; i++) { dot += h_dot[i]; }
-   return dot;
-}
-#endif // MFEM_USE_HIP
-
 real_t Vector::operator*(const Vector &v) const
 {
    MFEM_ASSERT(size == v.size, "incompatible Vectors!");
    if (size == 0) { return 0.0; }
 
    const bool use_dev = UseDevice() || v.UseDevice();
-#if defined(MFEM_USE_CUDA) || defined(MFEM_USE_HIP) || defined(MFEM_USE_OPENMP)
+
    auto m_data = Read(use_dev);
-#else
-   Read(use_dev);
-#endif
    auto v_data = v.Read(use_dev);
 
-   if (!use_dev) { goto vector_dot_cpu; }
-
+   if (use_dev)
+   {
+      // special path for OCCA and OpenMP
 #ifdef MFEM_USE_OCCA
-   if (DeviceCanUseOcca())
-   {
-      return occa::linalg::dot<real_t,real_t,real_t>(
-                OccaMemoryRead(data, size), OccaMemoryRead(v.data, size));
-   }
-#endif
-
-#ifdef MFEM_USE_CUDA
-   if (Device::Allows(Backend::CUDA_MASK))
-   {
-      return cuVectorDot(size, m_data, v_data);
-   }
-#endif
-
-#ifdef MFEM_USE_HIP
-   if (Device::Allows(Backend::HIP_MASK))
-   {
-      return hipVectorDot(size, m_data, v_data);
-   }
+      if (DeviceCanUseOcca())
+      {
+         return occa::linalg::dot<real_t, real_t, real_t>(
+                   OccaMemoryRead(data, size), OccaMemoryRead(v.data, size));
+      }
 #endif
 
 #ifdef MFEM_USE_OPENMP
-   if (Device::Allows(Backend::OMP_MASK))
-   {
+      if (Device::Allows(Backend::OMP_MASK))
+      {
 #define MFEM_USE_OPENMP_DETERMINISTIC_DOT
 #ifdef MFEM_USE_OPENMP_DETERMINISTIC_DOT
-      // By default, use a deterministic way of computing the dot product
-      static Vector th_dot;
-      #pragma omp parallel
-      {
-         const int nt = omp_get_num_threads();
-         #pragma omp master
-         th_dot.SetSize(nt);
-         const int tid    = omp_get_thread_num();
-         const int stride = (size + nt - 1)/nt;
-         const int start  = tid*stride;
-         const int stop   = std::min(start + stride, size);
-         real_t my_dot = 0.0;
-         for (int i = start; i < stop; i++)
+         // By default, use a deterministic way of computing the dot product
+         static Vector th_dot;
+         #pragma omp parallel
          {
-            my_dot += m_data[i] * v_data[i];
+            const int nt = omp_get_num_threads();
+            #pragma omp master
+            th_dot.SetSize(nt);
+            const int tid = omp_get_thread_num();
+            const int stride = (size + nt - 1) / nt;
+            const int start = tid * stride;
+            const int stop = std::min(start + stride, size);
+            real_t my_dot = 0.0;
+            for (int i = start; i < stop; i++)
+            {
+               my_dot += m_data[i] * v_data[i];
+            }
+            #pragma omp barrier
+            th_dot(tid) = my_dot;
          }
-         #pragma omp barrier
-         th_dot(tid) = my_dot;
-      }
-      return th_dot.Sum();
+         return th_dot.Sum();
 #else
-      // The standard way of computing the dot product is non-deterministic
-      real_t prod = 0.0;
-      #pragma omp parallel for reduction(+:prod)
-      for (int i = 0; i < size; i++)
-      {
-         prod += m_data[i] * v_data[i];
-      }
-      return prod;
+         // The standard way of computing the dot product is non-deterministic
+         real_t prod = 0.0;
+         #pragma omp parallel for reduction(+ : prod)
+         for (int i = 0; i < size; i++)
+         {
+            prod += m_data[i] * v_data[i];
+         }
+         return prod;
 #endif // MFEM_USE_OPENMP_DETERMINISTIC_DOT
-   }
+      }
 #endif // MFEM_USE_OPENMP
-   if (Device::Allows(Backend::DEBUG_DEVICE))
-   {
-      const int N = size;
-      auto v_data_ = v.Read();
-      auto m_data_ = Read();
-      Vector dot(1);
-      dot.UseDevice(true);
-      auto d_dot = dot.Write();
-      dot = 0.0;
-      mfem::forall(N, [=] MFEM_HOST_DEVICE (int i)
-      {
-         d_dot[0] += m_data_[i] * v_data_[i];
-      });
-      dot.HostReadWrite();
-      return dot[0];
    }
-vector_dot_cpu:
-   return operator*(v_data);
+
+   // normal path for everything else (cuda, hip, debug, cpu)
+   real_t res = 0;
+   reduce(
+      size, res,
+   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += m_data[i] * v_data[i]; },
+   SumReducer<real_t> {}, use_dev, vector_workspace());
+   return res;
 }
 
 real_t Vector::Min() const
@@ -1216,113 +1127,90 @@ real_t Vector::Min() const
    const bool use_dev = UseDevice();
    auto m_data = Read(use_dev);
 
-   if (!use_dev) { goto vector_min_cpu; }
+   if (use_dev)
+   {
+      // special case for OCCA and OpenMP
 
 #ifdef MFEM_USE_OCCA
-   if (DeviceCanUseOcca())
-   {
-      return occa::linalg::min<real_t,real_t>(OccaMemoryRead(data, size));
-   }
-#endif
-
-#ifdef MFEM_USE_CUDA
-   if (Device::Allows(Backend::CUDA_MASK))
-   {
-      return cuVectorMin(size, m_data);
-   }
-#endif
-
-#ifdef MFEM_USE_HIP
-   if (Device::Allows(Backend::HIP_MASK))
-   {
-      return hipVectorMin(size, m_data);
-   }
+      if (DeviceCanUseOcca())
+      {
+         return occa::linalg::min<real_t,real_t>(OccaMemoryRead(data, size));
+      }
 #endif
 
 #ifdef MFEM_USE_OPENMP
-   if (Device::Allows(Backend::OMP_MASK))
-   {
-      real_t minimum = m_data[0];
-      #pragma omp parallel for reduction(min:minimum)
-      for (int i = 0; i < size; i++)
+      if (Device::Allows(Backend::OMP_MASK))
       {
-         minimum = std::min(minimum, m_data[i]);
+         real_t minimum = m_data[0];
+         #pragma omp parallel for reduction(min:minimum)
+         for (int i = 0; i < size; i++)
+         {
+            minimum = std::min(minimum, m_data[i]);
+         }
+         return minimum;
       }
-      return minimum;
+#endif
    }
+
+   // normal path for everything else (cuda, hip, debug, cpu)
+   real_t res = infinity();
+   reduce(
+      size, res,
+   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmin(r, m_data[i]); },
+   MinReducer<real_t> {}, use_dev, vector_workspace());
+   return res;
+}
+
+real_t Vector::Max() const
+{
+   if (size == 0) { return -infinity(); }
+
+   const bool use_dev = UseDevice();
+   auto m_data = Read(use_dev);
+
+   if (use_dev)
+   {
+      // special cases where OCCA or OenMP are used
+#ifdef MFEM_USE_OCCA
+      if (DeviceCanUseOcca())
+      {
+         return occa::linalg::max<real_t, real_t>(OccaMemoryRead(data, size));
+      }
 #endif
 
-   if (Device::Allows(Backend::DEBUG_DEVICE))
-   {
-      const int N = size;
-      auto m_data_ = Read();
-      Vector min(1);
-      min = infinity();
-      min.UseDevice(true);
-      auto d_min = min.ReadWrite();
-      mfem::forall(N, [=] MFEM_HOST_DEVICE (int i)
+#ifdef MFEM_USE_OPENMP
+      if (Device::Allows(Backend::OMP_MASK))
       {
-         d_min[0] = (d_min[0]<m_data_[i])?d_min[0]:m_data_[i];
-      });
-      min.HostReadWrite();
-      return min[0];
+         real_t maximum = m_data[0];
+         #pragma omp parallel for reduction(max : maximum)
+         for (int i = 0; i < size; i++)
+         {
+            maximum = fmax(maximum, m_data[i]);
+         }
+         return maximum;
+      }
+#endif
    }
 
-vector_min_cpu:
-   real_t minimum = data[0];
-   for (int i = 1; i < size; i++)
-   {
-      if (m_data[i] < minimum)
-      {
-         minimum = m_data[i];
-      }
-   }
-   return minimum;
+   // normal path for everything else (cuda, hip, debug, cpu)
+   real_t res = -infinity();
+   reduce(
+      size, res,
+   [=] MFEM_HOST_DEVICE(int i, real_t &r) { r = fmax(r, m_data[i]); },
+   MaxReducer<real_t> {}, use_dev, vector_workspace());
+   return res;
 }
 
 real_t Vector::Sum() const
 {
    if (size == 0) { return 0.0; }
 
-   if (UseDevice())
-   {
-#ifdef MFEM_USE_CUDA
-      if (Device::Allows(Backend::CUDA_MASK))
-      {
-         return cuVectorDot(size, Read(), nullptr);
-      }
-#endif
-#ifdef MFEM_USE_HIP
-      if (Device::Allows(Backend::HIP_MASK))
-      {
-         return hipVectorDot(size, Read(), nullptr);
-      }
-#endif
-      if (Device::Allows(Backend::DEBUG_DEVICE))
-      {
-         const int N = size;
-         auto d_data = Read();
-         Vector sum(1);
-         sum.UseDevice(true);
-         auto d_sum = sum.Write();
-         d_sum[0] = 0.0;
-         mfem::forall(N, [=] MFEM_HOST_DEVICE (int i)
-         {
-            d_sum[0] += d_data[i];
-         });
-         sum.HostReadWrite();
-         return sum[0];
-      }
-   }
-
-   // CPU fallback
-   const real_t *h_data = HostRead();
-   real_t sum = 0.0;
-   for (int i = 0; i < size; i++)
-   {
-      sum += h_data[i];
-   }
-   return sum;
+   auto m_data = Read(UseDevice());
+   real_t res = 0;
+   reduce(
+   size, res, [=] MFEM_HOST_DEVICE(int i, real_t &r) { r += m_data[i]; },
+   SumReducer<real_t> {}, UseDevice(), vector_workspace());
+   return res;
 }
 
 }
