@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -28,7 +28,8 @@ namespace mfem
 
 using namespace bin_io;
 
-ParNCMesh::ParNCMesh(MPI_Comm comm, const NCMesh &ncmesh, int *part)
+ParNCMesh::ParNCMesh(MPI_Comm comm, const NCMesh &ncmesh,
+                     const int *partitioning)
    : NCMesh(ncmesh)
 {
    MyComm = comm;
@@ -39,7 +40,8 @@ ParNCMesh::ParNCMesh(MPI_Comm comm, const NCMesh &ncmesh, int *part)
    // sequence of leaf elements into 'NRanks' parts
    for (int i = 0; i < leaf_elements.Size(); i++)
    {
-      elements[leaf_elements[i]].rank = part ? part[i] : InitialPartition(i);
+      elements[leaf_elements[i]].rank =
+         partitioning ? partitioning[i] : InitialPartition(i);
    }
 
    Update();
@@ -53,6 +55,9 @@ ParNCMesh::ParNCMesh(MPI_Comm comm, std::istream &input, int version,
                      int &curved, int &is_nc)
    : NCMesh(input, version, curved, is_nc)
 {
+   MFEM_VERIFY(version != 11, "Nonconforming mesh format \"MFEM NC mesh v1.1\""
+               " is supported only in serial.");
+
    MyComm = comm;
    MPI_Comm_size(MyComm, &NRanks);
 
@@ -71,7 +76,7 @@ ParNCMesh::ParNCMesh(MPI_Comm comm, std::istream &input, int version,
                "size is not supported.");
 
    bool iso = Iso;
-   MPI_Allreduce(&iso, &Iso, 1, MPI_C_BOOL, MPI_LAND, MyComm);
+   MPI_Allreduce(&iso, &Iso, 1, MFEM_MPI_CXX_BOOL, MPI_LAND, MyComm);
 
    Update();
 }
@@ -248,6 +253,66 @@ void ParNCMesh::BuildEdgeList()
    // NOTE: entity_index_rank[1] is not deleted until CalculatePMatrixGroups
 }
 
+void ParNCMesh::FindEdgesOfGhostFace(int face, Array<int> & edges)
+{
+   const NCList &faceList = GetFaceList();
+   NCList::MeshIdAndType midt = faceList.GetMeshIdAndType(face);
+   if (!midt.id)
+   {
+      edges.SetSize(0);
+      return;
+   }
+
+   int V[4], E[4], Eo[4];
+   const int nfv = GetFaceVerticesEdges(*midt.id, V, E, Eo);
+   MFEM_ASSERT(nfv == 4, "");
+
+   edges.SetSize(nfv);
+   for (int i=0; i<nfv; ++i)
+   {
+      edges[i] = E[i];
+   }
+}
+
+void ParNCMesh::FindEdgesOfGhostElement(int elem, Array<int> & edges)
+{
+   NCMesh::Element &el = elements[elem]; // ghost element
+   MFEM_ASSERT(el.rank != MyRank, "");
+
+   MFEM_ASSERT(!el.ref_type, "not a leaf element.");
+
+   GeomInfo& gi = GI[el.Geom()];
+   edges.SetSize(gi.ne);
+
+   for (int j = 0; j < gi.ne; j++)
+   {
+      // get node for this edge
+      const int* ev = gi.edges[j];
+      int node[2] = { el.node[ev[0]], el.node[ev[1]] };
+
+      int enode = nodes.FindId(node[0], node[1]);
+      MFEM_ASSERT(enode >= 0, "edge node not found!");
+
+      Node &nd = nodes[enode];
+      MFEM_ASSERT(nd.HasEdge(), "edge not found!");
+
+      edges[j] = nd.edge_index;
+   }
+}
+
+void ParNCMesh::FindFacesOfGhostElement(int elem, Array<int> & faces)
+{
+   NCMesh::Element &el = elements[elem]; // ghost element
+   MFEM_ASSERT(el.rank != MyRank, "");
+   MFEM_ASSERT(!el.ref_type, "not a leaf element.");
+
+   faces.SetSize(GI[el.Geom()].nf);
+   for (int j = 0; j < faces.Size(); j++)
+   {
+      faces[j] = GetFace(el, j)->index;
+   }
+}
+
 void ParNCMesh::ElementSharesVertex(int elem, int local, int vnode)
 {
    // Analogous to ElementSharesEdge.
@@ -739,7 +804,7 @@ void ParNCMesh::ElementNeighborProcessors(int elem, Array<int> &ranks)
 template<class T>
 static void set_to_array(const std::set<T> &set, Array<T> &array)
 {
-   array.Reserve(set.size());
+   array.Reserve(static_cast<int>(set.size()));
    array.SetSize(0);
    for (auto x : set)
    {
@@ -840,7 +905,7 @@ void ParNCMesh::GetConformingSharedStructures(ParMesh &pmesh)
    }
 
    // create ParMesh groups, and the map (ncmesh_group -> pmesh_group)
-   Array<int> group_map(groups.size());
+   Array<int> group_map(static_cast<int>(groups.size()));
    {
       group_map = 0;
       IntegerSet iset;
@@ -849,7 +914,7 @@ void ParNCMesh::GetConformingSharedStructures(ParMesh &pmesh)
       {
          if (groups[i].size() > 1 || !i) // skip singleton groups
          {
-            iset.Recreate(groups[i].size(), groups[i].data());
+            iset.Recreate(static_cast<int>(groups[i].size()), groups[i].data());
             group_map[i] = int_groups.Insert(iset);
          }
       }
@@ -937,14 +1002,16 @@ void ParNCMesh::GetFaceNeighbors(ParMesh &pmesh)
    std::map<int, std::vector<int>> recv_elems;
 
    // Counts the number of slave faces of a master. This may be larger than the
-   // number of shared slaves if there exist degenerate slave-faces from face-edge constraints.
+   // number of shared slaves if there exist degenerate slave-faces from
+   // face-edge constraints.
    auto count_slaves = [&](int i, const Master& x)
    {
       return i + (x.slaves_end - x.slaves_begin);
    };
 
    const int bound = shared.conforming.Size() + std::accumulate(
-                        shared.masters.begin(), shared.masters.end(), 0, count_slaves);
+                        shared.masters.begin(), shared.masters.end(),
+                        0, count_slaves);
 
    fnbr.Reserve(bound);
    send_elems.Reserve(bound);
@@ -1050,7 +1117,7 @@ void ParNCMesh::GetFaceNeighbors(ParMesh &pmesh)
       for (int k = 0; k < gi.nv; k++)
       {
          int &v = vert_map[elem->node[k]];
-         if (!v) { v = vert_map.size(); }
+         if (!v) { v = static_cast<int>(vert_map.size()); }
          fne->GetVertices()[k] = v-1;
       }
 
@@ -1066,7 +1133,7 @@ void ParNCMesh::GetFaceNeighbors(ParMesh &pmesh)
 
    // create vertices in 'face_nbr_vertices'
    {
-      pmesh.face_nbr_vertices.SetSize(vert_map.size());
+      pmesh.face_nbr_vertices.SetSize(static_cast<int>(vert_map.size()));
       if (coordinates.Size())
       {
          tmp_vertex = new TmpVertex[nodes.NumIds()]; // TODO: something cheaper?
@@ -1449,7 +1516,7 @@ void ParNCMesh::Refine(const Array<Refinement> &refinements)
    for (int i = 0; i < refinements.Size(); i++)
    {
       const Refinement &ref = refinements[i];
-      MFEM_VERIFY(ref.ref_type == 7 || Dim < 3,
+      MFEM_VERIFY(ref.GetType() == 7 || Dim < 3,
                   "anisotropic parallel refinement not supported yet in 3D.");
    }
    MFEM_VERIFY(Iso || Dim < 3,
@@ -1478,7 +1545,7 @@ void ParNCMesh::Refine(const Array<Refinement> &refinements)
       ElementNeighborProcessors(elem, ranks);
       for (int j = 0; j < ranks.Size(); j++)
       {
-         send_ref[ranks[j]].AddRefinement(elem, ref.ref_type);
+         send_ref[ranks[j]].AddRefinement(elem, ref.GetType());
       }
    }
 
@@ -1489,7 +1556,7 @@ void ParNCMesh::Refine(const Array<Refinement> &refinements)
    for (int i = 0; i < refinements.Size(); i++)
    {
       const Refinement &ref = refinements[i];
-      NCMesh::RefineElement(leaf_elements[ref.index], ref.ref_type);
+      NCMesh::RefineElement(leaf_elements[ref.index], ref.GetType());
    }
 
    // receive (ghost layer) refinements from all neighbors
@@ -2229,7 +2296,7 @@ void ParNCMesh::SendRebalanceDofs(int old_ndofs,
    {
       RebalanceDofMessage &msg = it->second;
       msg.dofs.clear();
-      int ne = msg.elem_ids.size();
+      int ne = static_cast<int>(msg.elem_ids.size());
       if (ne)
       {
          msg.dofs.reserve(old_element_dofs.RowSize(msg.elem_ids[0]) * ne * vdim);
@@ -2259,8 +2326,8 @@ void ParNCMesh::RecvRebalanceDofs(Array<int> &elements, Array<long> &dofs)
    for (it = recv_rebalance_dofs.begin(); it != recv_rebalance_dofs.end(); ++it)
    {
       RebalanceDofMessage &msg = it->second;
-      ne += msg.elem_ids.size();
-      nd += msg.dofs.size();
+      ne += static_cast<int>(msg.elem_ids.size());
+      nd += static_cast<int>(msg.dofs.size());
    }
 
    elements.SetSize(ne);
@@ -2478,7 +2545,7 @@ void ParNCMesh::AdjustMeshIds(Array<MeshId> ids[], int rank)
    if (!shared_edges.masters.Size() &&
        !shared_faces.masters.Size()) { return; }
 
-   Array<bool> contains_rank(groups.size());
+   Array<bool> contains_rank(static_cast<int>(groups.size()));
    for (unsigned i = 0; i < groups.size(); i++)
    {
       contains_rank[i] = GroupContains(i, rank);
@@ -2793,7 +2860,7 @@ void ParNCMesh::ElementValueMessage<ValueType, RefTypes, Tag>::Encode(int)
    std::ostringstream ostream;
 
    Array<int> tmp_elements;
-   tmp_elements.MakeRef(elements.data(), elements.size());
+   tmp_elements.MakeRef(elements.data(), static_cast<int>(elements.size()));
 
    ElementSet eset(pncmesh, RefTypes);
    eset.Encode(tmp_elements);
@@ -2810,7 +2877,7 @@ void ParNCMesh::ElementValueMessage<ValueType, RefTypes, Tag>::Encode(int)
       element_index[decoded[i]] = i;
    }
 
-   write<int>(ostream, values.size());
+   write<int>(ostream, static_cast<int>(values.size()));
    MFEM_ASSERT(elements.size() == values.size(), "");
 
    for (unsigned i = 0; i < values.size(); i++)
@@ -2868,7 +2935,7 @@ void ParNCMesh::RebalanceDofMessage::SetElements(const Array<int> &elems,
 
 static void write_dofs(std::ostream &os, const std::vector<int> &dofs)
 {
-   write<int>(os, dofs.size());
+   write<int>(os, static_cast<int>(dofs.size()));
    // TODO: we should compress the ints, mostly they are contiguous ranges
    os.write((const char*) dofs.data(), dofs.size() * sizeof(int));
 }
@@ -3049,6 +3116,79 @@ int ParNCMesh::PrintMemoryDetail(bool with_base) const
    return leaf_elements.Size();
 }
 
+void ParNCMesh::GetGhostElements(Array<int> & gelem)
+{
+   gelem.SetSize(NGhostElements);
+
+   for (int g=0; g<NGhostElements; ++g)
+   {
+      // This is an index in NCMesh::elements, an array of all elements, cf.
+      // NCMesh::OnMeshUpdated.
+      gelem[g] = leaf_elements[NElements + g];
+   }
+}
+
+// Note that this function is modeled after ParNCMesh::Refine().
+void ParNCMesh::CommunicateGhostData(
+   const Array<VarOrderElemInfo> & sendData, Array<VarOrderElemInfo> & recvData)
+{
+   recvData.SetSize(0);
+
+   if (NRanks == 1) { return; }
+
+   NeighborPRefinementMessage::Map send_ref;
+
+   // create refinement messages to all neighbors (NOTE: some may be empty)
+   Array<int> neighbors;
+   NeighborProcessors(neighbors);
+   for (int i = 0; i < neighbors.Size(); i++)
+   {
+      send_ref[neighbors[i]].SetNCMesh(this);
+   }
+
+   // populate messages: all refinements that occur next to the processor
+   // boundary need to be sent to the adjoining neighbors so they can keep
+   // their ghost layer up to date
+   Array<int> ranks;
+   ranks.Reserve(64);
+   for (int i = 0; i < sendData.Size(); i++)
+   {
+      MFEM_ASSERT(sendData[i].element < (unsigned int) NElements, "");
+      const int elem = leaf_elements[sendData[i].element];
+      ElementNeighborProcessors(elem, ranks);
+      for (int j = 0; j < ranks.Size(); j++)
+      {
+         send_ref[ranks[j]].AddRefinement(elem, sendData[i].order);
+      }
+   }
+
+   // send the messages (overlap with local refinements)
+   NeighborPRefinementMessage::IsendAll(send_ref, MyComm);
+
+   // receive (ghost layer) refinements from all neighbors
+   for (int j = 0; j < neighbors.Size(); j++)
+   {
+      int rank, size;
+      NeighborPRefinementMessage::Probe(rank, size, MyComm);
+
+      NeighborPRefinementMessage msg;
+      msg.SetNCMesh(this);
+      msg.Recv(rank, size, MyComm);
+
+      // Get the ghost refinement data
+      const int os = recvData.Size();
+      recvData.SetSize(os + msg.Size());
+      for (int i = 0; i < msg.Size(); i++)
+      {
+         recvData[os + i].element = msg.elements[i];
+         recvData[os + i].order = msg.values[i];
+      }
+   }
+
+   // make sure we can delete the send buffers
+   NeighborPRefinementMessage::WaitAllSent(send_ref);
+}
+
 } // namespace mfem
 
 #endif // MFEM_USE_MPI
