@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -22,13 +22,11 @@
 #include "ncmesh.hpp"
 #include "../general/communication.hpp"
 #include "../general/sort_pairs.hpp"
+#include "../fem/fespace.hpp"
 
 namespace mfem
 {
 
-class FiniteElementSpace;
-
-
 /** \brief A parallel extension of the NCMesh class.
  *
  *  The basic idea (and assumption) is that all processors share the coarsest
@@ -63,11 +61,14 @@ class FiniteElementSpace;
  */
 class ParNCMesh : public NCMesh
 {
+protected:
+   ParNCMesh() = default;
 public:
    /// Construct by partitioning a serial NCMesh.
    /** SFC partitioning is used by default. A user-specified partition can be
        passed in 'part', where part[i] is the desired MPI rank for element i. */
-   ParNCMesh(MPI_Comm comm, const NCMesh& ncmesh, int* part = NULL);
+   ParNCMesh(MPI_Comm comm, const NCMesh& ncmesh,
+             const int *partitioning = nullptr);
 
    /** Load from a stream, parallel version. See the serial NCMesh::NCMesh
        counterpart for a description of the parameters. */
@@ -81,7 +82,8 @@ public:
 
    /** An override of NCMesh::Refine, which is called eventually, after making
        sure that refinements that occur on the processor boundary are sent to
-       the neighbor processors so they can keep their ghost layers up to date.*/
+       the neighbor processors so they can keep their ghost layers up to
+       date. */
    void Refine(const Array<Refinement> &refinements) override;
 
    /// Parallel version of NCMesh::LimitNCLevel.
@@ -108,7 +110,7 @@ public:
        passed. */
    void Rebalance(const Array<int> *custom_partition = NULL);
 
-   // interface for ParFiniteElementSpace
+   // Interface for ParFiniteElementSpace
    int GetNElements() const { return NElements; }
 
    int GetNGhostVertices() const { return NGhostVertices; }
@@ -122,6 +124,10 @@ public:
    const NCList& GetSharedEdges() { GetEdgeList(); return shared_edges; }
    const NCList& GetSharedFaces() { GetFaceList(); return shared_faces; }
 
+   /* Gets the array of global serial indices in NCMesh::elements of all ghost
+      elements for this processor. */
+   void GetGhostElements(Array<int> & gelem);
+
    /// Helper to get shared vertices/edges/faces ('entity' == 0/1/2 resp.).
    const NCList& GetSharedList(int entity)
    {
@@ -229,8 +235,8 @@ public:
                                     const Table &deref_table);
 
    /** Extension of NCMesh::GetBoundaryClosure. Filters out ghost vertices and
-       ghost edges from 'bdr_vertices' and 'bdr_edges', and uncovers hidden internal
-       boundary faces. */
+       ghost edges from 'bdr_vertices' and 'bdr_edges', and uncovers hidden
+       internal boundary faces. */
    void GetBoundaryClosure(const Array<int> &bdr_attr_is_ess,
                            Array<int> &bdr_vertices,
                            Array<int> &bdr_edges, Array<int> &bdr_faces) override;
@@ -247,13 +253,38 @@ public:
        The debug mesh will have element attributes set to element rank + 1. */
    void GetDebugMesh(Mesh &debug_mesh) const;
 
+   // Ghost element indices and their orders, for parallel p-refinement.
+   struct VarOrderElemInfo
+   {
+      unsigned int element; // Element index
+      char order; // Element order
+   };
+
+   void CommunicateGhostData(
+      const Array<VarOrderElemInfo> & sendData,
+      Array<VarOrderElemInfo> & recvData);
+
+   /** For a ghost element with index @a elem, the edge indices are returned in
+       @a edges. */
+   void FindEdgesOfGhostElement(int elem, Array<int> & edges);
+
+   /** For a ghost element with index @a elem, the face indices are returned in
+       @a faces. */
+   void FindFacesOfGhostElement(int elem, Array<int> & faces);
+
+   /** For a ghost face with index @a face, the edge indices are returned in
+       @a edges. */
+   void FindEdgesOfGhostFace(int face, Array<int> & edges);
+
 protected: // interface for ParMesh
 
    friend class ParMesh;
+   friend class ParSubMesh;
 
    /** For compatibility with conforming code in ParMesh and ParFESpace.
-       Initializes shared structures in ParMesh: gtopo, shared_*, group_s*, s*_l*.
-       The ParMesh then acts as a parallel mesh cut along the NC interfaces. */
+       Initializes shared structures in ParMesh: gtopo, shared_*, group_s*,
+       s*_l*. The ParMesh then acts as a parallel mesh cut along the NC
+       interfaces. */
    void GetConformingSharedStructures(class ParMesh &pmesh);
 
    /** Populate face neighbor members of ParMesh from the ghost layer, without
@@ -435,7 +466,7 @@ protected: // implementation
       std::vector<int> elements;
       std::vector<ValueType> values;
 
-      int Size() const { return elements.size(); }
+      int Size() const { return static_cast<int>(elements.size()); }
       void Reserve(int size) { elements.reserve(size); values.reserve(size); }
 
       void Add(int elem, ValueType val)
@@ -456,7 +487,8 @@ protected: // implementation
    /** Used by ParNCMesh::Refine() to inform neighbors about refinements at
     *  the processor boundary. This keeps their ghost layers synchronized.
     */
-   class NeighborRefinementMessage : public ElementValueMessage<char, false, 289>
+   class NeighborRefinementMessage : public ElementValueMessage<char, false,
+      VarMessageTag::NEIGHBOR_REFINEMENT_VM>
    {
    public:
       void AddRefinement(int elem, char ref_type) { Add(elem, ref_type); }
@@ -465,7 +497,8 @@ protected: // implementation
 
    /** Used by ParNCMesh::Derefine() to keep the ghost layers synchronized.
     */
-   class NeighborDerefinementMessage : public ElementValueMessage<int, false, 290>
+   class NeighborDerefinementMessage : public ElementValueMessage<int, false,
+      VarMessageTag::NEIGHBOR_DEREFINEMENT_VM>
    {
    public:
       void AddDerefinement(int elem, int rank) { Add(elem, rank); }
@@ -475,7 +508,8 @@ protected: // implementation
    /** Used in Step 2 of Rebalance() to synchronize new rank assignments in
     *  the ghost layer.
     */
-   class NeighborElementRankMessage : public ElementValueMessage<int, false, 156>
+   class NeighborElementRankMessage : public ElementValueMessage<int, false,
+      VarMessageTag::NEIGHBOR_ELEMENT_RANK_VM>
    {
    public:
       void AddElementRank(int elem, int rank) { Add(elem, rank); }
@@ -486,7 +520,8 @@ protected: // implementation
     *  RefTypes == true which means the refinement hierarchy will be recreated
     *  on the receiving side.
     */
-   class RebalanceMessage : public ElementValueMessage<int, true, 157>
+   class RebalanceMessage : public ElementValueMessage<int, true,
+      VarMessageTag::REBALANCE_VM>
    {
    public:
       void AddElementRank(int elem, int rank) { Add(elem, rank); }
@@ -496,7 +531,7 @@ protected: // implementation
    /** Allows migrating element data (DOFs) after Rebalance().
     *  Used by SendRebalanceDofs and RecvRebalanceDofs.
     */
-   class RebalanceDofMessage : public VarMessage<158>
+   class RebalanceDofMessage : public VarMessage<VarMessageTag::REBALANCE_DOF_VM>
    {
    public:
       std::vector<int> elem_ids, dofs;
@@ -515,12 +550,23 @@ protected: // implementation
       void Decode(int) override;
    };
 
+   /** Used by CommunicateGhostData to send p-refined element indices and
+       their order. */
+   class NeighborPRefinementMessage : public ElementValueMessage<int, false,
+      VarMessageTag::NEIGHBOR_PREFINEMENT_VM>
+   {
+   public:
+      void AddRefinement(int elem, int order) { Add(elem, order); }
+      typedef std::map<int, NeighborPRefinementMessage> Map;
+   };
+
    /** Assign new Element::rank to leaf elements and send them to their new
        owners, keeping the ghost layer up to date. Used by Rebalance() and
        Derefine(). 'target_elements' is the number of elements this rank
        is supposed to own after the exchange. If this number is not known
-       a priori, the parameter can be set to -1, but more expensive communication
-       (synchronous sends and a barrier) will be used in that case. */
+       a priori, the parameter can be set to -1, but more expensive
+       communication (synchronous sends and a barrier) will be used in that
+       case. */
    void RedistributeElements(Array<int> &new_ranks, int target_elements,
                              bool record_comm);
 
@@ -541,10 +587,10 @@ protected: // implementation
    std::size_t GroupsMemoryUsage() const;
 
    friend class NeighborRowMessage;
+   friend class NeighborOrderMessage;
 };
 
 
-
 // comparison operator so that MeshId can be used as key in std::map
 inline bool operator< (const NCMesh::MeshId &a, const NCMesh::MeshId &b)
 {
