@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -31,15 +31,27 @@ namespace mfem
 class TransferMap
 {
 public:
+   /**
+    * @brief Construct a new TransferMap object which transfers degrees of
+    * freedom from the source FiniteElementSpace to the destination
+    * FiniteElementSpace.
+    *
+    * @param src The source FiniteElementSpace
+    * @param dst The destination FiniteElementSpace
+    */
+   TransferMap(const FiniteElementSpace &src, const FiniteElementSpace &dst);
+
    /**
     * @brief Construct a new TransferMap object which transfers degrees of
     * freedom from the source GridFunction to the destination GridFunction.
     *
+    * Equivalent to creating the TransferMap from the finite element spaces of
+    * each of the GridFunction%s.
+    *
     * @param src The source GridFunction
     * @param dst The destination GridFunction
     */
-   TransferMap(const GridFunction &src,
-               const GridFunction &dst);
+   TransferMap(const GridFunction &src, const GridFunction &dst);
 
    /**
     * @brief Transfer the source GridFunction to the destination GridFunction.
@@ -62,27 +74,43 @@ private:
 
    /// Mapping of the GridFunction defined on the SubMesh to the GridFunction
    /// of its parent Mesh.
-   Array<int> sub1_to_parent_map_;
+   Array<int> sub_to_parent_map_;
 
-   /// Mapping of the GridFunction defined on the second SubMesh to the
-   /// GridFunction of its parent Mesh. This is only used if this TransferMap
-   /// represents a SubMesh to SubMesh transfer.
-   Array<int> sub2_to_parent_map_;
+   /// Pointer to the finite element space defined on the SubMesh.
+   const FiniteElementSpace *sub_fes_ = nullptr;
+
+   /// @name Needed for SubMesh-to-SubMesh transfer
+   ///@{
 
    /// Pointer to the supplemental FiniteElementSpace on the common root parent
    /// Mesh. This is only used if this TransferMap represents a SubMesh to
    /// SubMesh transfer.
-   std::unique_ptr<const FiniteElementSpace> root_fes_;
+   std::unique_ptr<FiniteElementSpace> root_fes_;
 
    /// Pointer to the supplemental FiniteElementCollection used with root_fes_.
-   /// This is only used if this TransferMap represents a SubMesh to
-   /// SubMesh transfer where the root requires a different type of collection
-   /// than the SubMesh objects. For example, when the subpaces are L2 on
-   /// boundaries of the parent mesh and the root space can be RT.
+   /// This is only used if this TransferMap represents a SubMesh to SubMesh
+   /// transfer where the root requires a different type of collection than the
+   /// SubMesh objects. For example, when the subpaces are L2 on boundaries of
+   /// the parent mesh and the root space can be RT.
    std::unique_ptr<const FiniteElementCollection> root_fec_;
 
+   /// Transfer mapping from the source to the parent (root).
+   std::unique_ptr<TransferMap> src_to_parent;
+
+   /// @brief Transfer mapping from the destination to the parent (root).
+   ///
+   /// SubMesh-to-SubMesh transfer works by bringing both the source and
+   /// destination data to their common parent, and then transferring back to
+   /// the destination.
+   std::unique_ptr<TransferMap> dst_to_parent;
+
+   /// Transfer mapping from the parent to the destination.
+   std::unique_ptr<TransferMap> parent_to_dst;
+
+   ///@}
+
    /// Temporary vector
-   mutable Vector z_;
+   mutable GridFunction z_;
 };
 
 } // namespace mfem
