@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -64,7 +64,7 @@ public:
 
    void InitializeGLVis();
 
-   void DisplayToGLVis();
+   void DisplayToGLVis(int visport = 19916);
 
    const ParGridFunction & GetVectorPotential() { return *a_; }
 
