@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -36,8 +36,8 @@
 //     mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 2 -tid 4 -ni 200 -bnd -qt 1 -qo 8
 //   Adapted analytic size+orientation:
 //     mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 14 -tid 4 -ni 100 -bnd -qt 1 -qo 8
-//   Adapted analytic shape+orientation:
-//     mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 85 -tid 4 -ni 100 -bnd -qt 1 -qo 8 -fd
+//   Adapted analytic shape+orientation (AD):
+//     mesh-optimizer -m square01.mesh -o 3 -rs 2 -mid 85 -tid 4 -ni 100 -bnd -qt 1 -qo 8 -rtol 1e-6
 //
 //   Adapted analytic shape and/or size with hr-adaptivity:
 //     mesh-optimizer -m square01.mesh -o 2 -tid 9  -ni 50 -li 20 -hmid 55 -mid 7 -hr
@@ -51,32 +51,38 @@
 //   Adapted discrete size NC mesh;
 //     mesh-optimizer -m amr-quad-q2.mesh -o 2 -rs 2 -mid 94 -tid 5 -ni 50 -qo 4 -nor
 //   Adapted discrete size 3D with PA:
-//     mesh-optimizer -m cube.mesh -o 2 -rs 2 -mid 321 -tid 5 -ls 3 -nor -pa
+//     mesh-optimizer -m cube.mesh -o 2 -rs 2 -mid 321 -tid 5 -ls 3 -nor -pa -rtol 1e-8
 //   Adapted discrete size 3D with PA on device (requires CUDA):
 //   * mesh-optimizer -m cube.mesh -o 3 -rs 3 -mid 321 -tid 5 -ls 3 -nor -lc 0.1 -pa -d cuda
 //   Adapted discrete size; explicit combo of metrics; mixed tri/quad mesh:
 //     mesh-optimizer -m ../../data/square-mixed.mesh -o 2 -rs 2 -mid 2 -tid 5 -ni 200 -bnd -qo 6 -cmb 2 -nor
 //   Adapted discrete size+aspect_ratio:
 //     mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 7 -tid 6 -ni 100
-//     mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 7 -tid 6 -ni 100 -qo 6 -ex -st 1 -nor
-//   Adapted discrete size+orientation:
-//      mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 36 -tid 8 -qo 4 -fd -nor
+//   Adapted discrete size+orientation (AD):
+//      mesh-optimizer -m square01.mesh -o 2 -rs 2 -mid 36 -tid 8 -qo 4 -nor -rtol 1e-6
 //   Adapted discrete aspect ratio (3D):
 //     mesh-optimizer -m cube.mesh -o 2 -rs 2 -mid 302 -tid 7 -ni 20 -bnd -qt 1 -qo 8
 //
+//   Periodic 2D + adapted discrete size:
+//     mesh-optimizer -m ../../data/periodic-square.mesh -o 2 -rs 4 -mid 94 -tid 5 -qo 4 -nor
+//     mesh-optimizer -m periodic-tri.mesh -o 2 -rs 4 -mid 94 -tid 5 -qo 4 -nor
+//   Periodic 3D + adapted discrete size + PA:
+//     mesh-optimizer -m periodic-cube.mesh -o 2 -rs 2 -mid 338 -tid 5 -nor -rtol 1e-6 -qo 4 -pa
+//   Periodic 2D NC mesh + adapted discrete size + PA:
+//     (the mesh is in the mfem/data GitHub repository)
+//   * mesh-optimizer -m ../../../data/periodic/per-amr-square.mesh -o 2 -mid 94 -tid 5 -ni 50 -qo 4 -nor -pa
+//
 //   Adaptive limiting:
 //     mesh-optimizer -m stretched2D.mesh -o 2 -mid 2 -tid 1 -ni 50 -qo 5 -nor -vl 1 -alc 0.5
 //   Adaptive limiting through the L-BFGS solver:
-//     mesh-optimizer -m stretched2D.mesh -o 2 -mid 2 -tid 1 -ni 400 -qo 5 -nor -vl 1 -alc 0.5 -st 1
-//   Adaptive limiting through FD (requires GSLIB):
-//   * mesh-optimizer -m stretched2D.mesh -o 2 -mid 2 -tid 1 -ni 50 -qo 5 -nor -vl 1 -alc 0.5 -fd -ae 1
+//     mesh-optimizer -m stretched2D.mesh -o 2 -mid 2 -tid 1 -ni 400 -qo 5 -nor -vl 1 -alc 0.5 -st 1 -rtol 1e-8
 //
 //   Blade shape:
 //     mesh-optimizer -m blade.mesh -o 4 -mid 2 -tid 1 -ni 30 -ls 3 -art 1 -bnd -qt 1 -qo 8
+//   Blade shape (AD):
+//     mesh-optimizer -m blade.mesh -o 4 -mid 11 -tid 1 -ni 30 -ls 3 -art 1 -bnd -qt 1 -qo 8
 //     (requires CUDA):
 //   * mesh-optimizer -m blade.mesh -o 4 -mid 2 -tid 1 -ni 30 -ls 3 -art 1 -bnd -qt 1 -qo 8 -d cuda
-//   Blade shape with FD-based solver:
-//     mesh-optimizer -m blade.mesh -o 4 -mid 2 -tid 1 -ni 30 -ls 4 -bnd -qt 1 -qo 8 -fd
 //   Blade limited shape:
 //     mesh-optimizer -m blade.mesh -o 4 -mid 2 -tid 1 -bnd -qt 1 -qo 8 -lc 5000
 //   ICF shape and equal size:
@@ -114,7 +120,7 @@ using namespace std;
 
 int main(int argc, char *argv[])
 {
-   // 0. Set the method's default parameters.
+   // Set the method's default parameters.
    const char *mesh_file = "icf.mesh";
    int mesh_poly_deg     = 1;
    int rs_levels         = 0;
@@ -151,11 +157,11 @@ int main(int argc, char *argv[])
    bool pa               = false;
    int n_hr_iter         = 5;
    int n_h_iter          = 1;
-   int mesh_node_ordering = 0;
-   int barrier_type       = 0;
-   int worst_case_type    = 0;
+   int mesh_node_order   = 0;
+   int barrier_type      = 0;
+   int worst_case_type   = 0;
 
-   // 1. Parse command-line options.
+   // Parse command-line options.
    OptionsParser args(argc, argv);
    args.AddOption(&mesh_file, "-m", "--mesh",
                   "Mesh file to use.");
@@ -185,7 +191,6 @@ int main(int argc, char *argv[])
                   "94 : balanced combo mu_2 & mu_56    -- 2D shape+size\n\t"
                   "98 : (1/tau)|T-I|^2                 -- 2D shape+size+orientation\n\t"
                   // "211: (tau-1)^2-tau+sqrt(tau^2+eps)  -- 2D untangling\n\t"
-                  // "211: (tau-1)^2-tau+sqrt(tau^2)      -- 2D untangling\n\t"
                   // "252: 0.5(tau-1)^2/(tau-tau_0)       -- 2D untangling\n\t"
                   "301: (|T||T^-1|)/3-1              -- 3D shape\n\t"
                   "302: (|T|^2|T^-1|^2)/9-1          -- 3D shape\n\t"
@@ -209,6 +214,8 @@ int main(int argc, char *argv[])
                   "A-metrics\n\t"
                   "11 : (1/4*alpha)|A-(adjA)^T(W^TW)/omega|^2 -- 2D shape\n\t"
                   "36 : (1/alpha)|A-W|^2                      -- 2D shape+size+orientation\n\t"
+                  "49 : (1-gamma) mu_2 + gamma nu_50          -- 2D shape+skew\n\t"
+                  "51 : see fem/tmop.hpp                      -- 2D size+skew\n\t"
                   "107: (1/2*alpha)|A-|A|/|W|W|^2             -- 2D shape+orientation\n\t"
                   "126: (1-gamma)nu_11 + gamma*nu_14a         -- 2D shape+size\n\t"
                  );
@@ -298,7 +305,7 @@ int main(int argc, char *argv[])
    args.AddOption(&n_h_iter, "-nh", "--n_h_iter",
                   "Number of h-adaptivity iterations per r-adaptivity"
                   "iteration.");
-   args.AddOption(&mesh_node_ordering, "-mno", "--mesh_node_ordering",
+   args.AddOption(&mesh_node_order, "-mno", "--mesh_node_ordering",
                   "Ordering of mesh nodes."
                   "0 (default): byNodes, 1: byVDIM");
    args.AddOption(&barrier_type, "-btype", "--barrier-type",
@@ -327,54 +334,57 @@ int main(int argc, char *argv[])
    Device device(devopt);
    device.Print();
 
-   // 2. Initialize and refine the starting mesh.
+   // Initialize and refine the starting mesh.
    Mesh *mesh = new Mesh(mesh_file, 1, 1, false);
    for (int lev = 0; lev < rs_levels; lev++) { mesh->UniformRefinement(); }
    const int dim = mesh->Dimension();
 
    if (hradaptivity) { mesh->EnsureNCMesh(); }
 
-   // 3. Define a finite element space on the mesh-> Here we use vector finite
-   //    elements which are tensor products of quadratic finite elements. The
-   //    number of components in the vector finite element space is specified by
-   //    the last parameter of the FiniteElementSpace constructor.
+   auto s = mesh->GetNodalFESpace();
+   const bool periodic = (s && s->IsDGSpace()) ? true : false;
+
+   // Define a FE space on the mesh, based on the input order. This space will
+   // also be used to represent the nodal positions of the mesh. We use a vector
+   // FE space which is a tensor product of a scalar FE space. The number of
+   // components in the vector finite element space matches the dimension.
    FiniteElementCollection *fec;
-   if (mesh_poly_deg <= 0)
+   if (mesh_poly_deg <= 0) { mesh_poly_deg = 2; }
+   if (periodic)
    {
-      fec = new QuadraticPosFECollection;
-      mesh_poly_deg = 2;
+      fec = new L2_FECollection(mesh_poly_deg, dim, BasisType::GaussLobatto);
    }
    else { fec = new H1_FECollection(mesh_poly_deg, dim); }
-   FiniteElementSpace *fespace = new FiniteElementSpace(mesh, fec, dim,
-                                                        mesh_node_ordering);
+   auto fespace = new FiniteElementSpace(mesh, fec, dim, mesh_node_order);
 
-   // 4. Make the mesh curved based on the above finite element space. This
-   //    means that we define the mesh elements through a fespace-based
-   //    transformation of the reference element.
+   // Make the starting mesh curved. This means we define the mesh elements
+   // through a FE-based transformation of the reference element.
    mesh->SetNodalFESpace(fespace);
 
-   // 5. Set up an empty right-hand side vector b, which is equivalent to b=0.
-   Vector b(0);
-
-   // 6. Get the mesh nodes (vertices and other degrees of freedom in the finite
-   //    element space) as a finite element grid function in fespace. Note that
-   //    changing x automatically changes the shapes of the mesh elements.
+   // Get the mesh nodes (vertices and other DOFs in the FE space) as a FE grid
+   // function in pfespace. Note that changing x automatically changes the
+   // shapes of the mesh elements.
    GridFunction x(fespace);
    mesh->SetNodalGridFunction(&x);
 
-   // 7. Define a vector representing the minimal local mesh size in the mesh
-   //    nodes. We index the nodes using the scalar version of the degrees of
-   //    freedom in fespace. Note: this is partition-dependent.
-   //
-   //    In addition, compute average mesh size and total volume.
-   Vector h0(fespace->GetNDofs());
+   // We create an H1 space for the mesh displacement. The displacement is
+   // always in a continuous space, even if the mesh is periodic.
+   // The nonlinear problem will be solved for the continuous displacement.
+   H1_FECollection fec_h1(mesh_poly_deg, dim);
+   FiniteElementSpace fes_h1(mesh, &fec_h1, dim,  mesh_node_order);
+   GridFunction dx(&fes_h1); dx = 0.0;
+
+   // Define a vector representing the minimal local mesh size in the mesh
+   // nodes. We index the nodes by the scalar version of the DOFs in pfespace.
+   // In addition, compute average mesh size and total volume.
+   Vector h0(fes_h1.GetNDofs());
    h0 = infinity();
    real_t mesh_volume = 0.0;
    Array<int> dofs;
    for (int i = 0; i < mesh->GetNE(); i++)
    {
       // Get the local scalar element degrees of freedom in dofs.
-      fespace->GetElementDofs(i, dofs);
+      fes_h1.GetElementDofs(i, dofs);
       // Adjust the value of h0 in dofs based on the local mesh size.
       const real_t hi = mesh->GetElementSize(i);
       for (int j = 0; j < dofs.Size(); j++)
@@ -385,48 +395,69 @@ int main(int argc, char *argv[])
    }
    const real_t small_phys_size = pow(mesh_volume, 1.0 / dim) / 100.0;
 
-   // 8. Add a random perturbation to the nodes in the interior of the domain.
-   //    We define a random grid function of fespace and make sure that it is
-   //    zero on the boundary and its values are locally of the order of h0.
-   //    The latter is based on the DofToVDof() method which maps the scalar to
-   //    the vector degrees of freedom in fespace.
-   GridFunction rdm(fespace);
-   rdm.Randomize();
-   rdm -= 0.25; // Shift to random values in [-0.5,0.5].
-   rdm *= jitter;
-   rdm.HostReadWrite();
-   // Scale the random values to be of order of the local mesh size.
-   for (int i = 0; i < fespace->GetNDofs(); i++)
+   // Add a random perturbation to the nodes in the interior of the domain.
+   // We define a random grid function of pfespace and make sure that it is
+   // zero on the boundary and its values are locally of the order of h0.
+   // The latter is based on the DofToVDof() method which maps the scalar to
+   // the vector degrees of freedom in pfespace.
+   if (jitter > 0)
    {
-      for (int d = 0; d < dim; d++)
+      GridFunction rdm(&fes_h1);
+      rdm.Randomize();
+      rdm -= 0.25; // Shift to random values in [-0.5,0.5].
+      rdm *= jitter;
+      rdm.HostReadWrite();
+      // Scale the random values to be of order of the local mesh size.
+      for (int i = 0; i < fes_h1.GetNDofs(); i++)
       {
-         rdm(fespace->DofToVDof(i,d)) *= h0(i);
+         for (int d = 0; d < dim; d++)
+         {
+            rdm(fes_h1.DofToVDof(i,d)) *= h0(i);
+         }
+      }
+      // Set the boundary values to zero. Note that periodic periodic boundaries
+      // will be free to move.
+      Array<int> vdofs;
+      for (int i = 0; i < fes_h1.GetNBE(); i++)
+      {
+         fespace->GetBdrElementVDofs(i, vdofs);
+         for (int j = 0; j < vdofs.Size(); j++) { rdm(vdofs[j]) = 0.0; }
+      }
+
+      if (periodic)
+      {
+         // For H1 the perturbation is controlled by the true nodes.
+         rdm.SetFromTrueVector();
+         GridFunction rdm_l2(fespace);
+         rdm_l2.ProjectGridFunction(rdm);
+         x -= rdm_l2;
+      }
+      else
+      {
+         x -= rdm;
+         // For H1 the perturbation is controlled by the true nodes.
+         x.SetFromTrueVector();
+      }
+
+      if (visualization)
+      {
+         socketstream vis1;
+         common::VisualizeMesh(vis1, "localhost", 19916, *mesh, "Perturbed",
+                               300, 600, 300, 300);
       }
    }
-   Array<int> vdofs;
-   for (int i = 0; i < fespace->GetNBE(); i++)
-   {
-      // Get the vector degrees of freedom in the boundary element.
-      fespace->GetBdrElementVDofs(i, vdofs);
-      // Set the boundary values to zero.
-      for (int j = 0; j < vdofs.Size(); j++) { rdm(vdofs[j]) = 0.0; }
-   }
-   x -= rdm;
-   x.SetTrueVector();
-   x.SetFromTrueVector();
 
-   // 9. Save the starting (prior to the optimization) mesh to a file. This
-   //    output can be viewed later using GLVis: "glvis -m perturbed.mesh".
+   // Save the starting (prior to the optimization) mesh to a file. This
+   // output can be viewed later using GLVis: "glvis -m perturbed.mesh".
    {
       ofstream mesh_ofs("perturbed.mesh");
       mesh->Print(mesh_ofs);
    }
 
-   // 10. Store the starting (prior to the optimization) positions.
-   GridFunction x0(fespace);
-   x0 = x;
+   // Store the starting (prior to the optimization) positions.
+   GridFunction x0(x);
 
-   // 11. Form the integrator that uses the chosen metric and target.
+   // Form the integrator that uses the chosen metric and target.
    real_t min_detJ = -0.1;
    TMOP_QualityMetric *metric = NULL;
    switch (metric_id)
@@ -474,7 +505,9 @@ int main(int argc, char *argv[])
       // A-metrics
       case 11: metric = new TMOP_AMetric_011; break;
       case 36: metric = new TMOP_AMetric_036; break;
-      case 107: metric = new TMOP_AMetric_107a; break;
+      case 49: metric = new TMOP_AMetric_049(0.9); break;
+      case 51: metric = new TMOP_AMetric_051; break;
+      case 107: metric = new TMOP_AMetric_107; break;
       case 126: metric = new TMOP_AMetric_126(0.9); break;
       default:
          cout << "Unknown metric_id: " << metric_id << endl;
@@ -558,7 +591,9 @@ int main(int argc, char *argv[])
    TargetConstructor *target_c = NULL;
    HessianCoefficient *adapt_coeff = NULL;
    HRHessianCoefficient *hr_adapt_coeff = NULL;
-   H1_FECollection ind_fec(mesh_poly_deg, dim);
+   int ind_fec_order = (target_id >= 5 && target_id <= 8 && !fdscheme) ?
+                       1 : mesh_poly_deg;
+   H1_FECollection ind_fec(ind_fec_order, dim);
    FiniteElementSpace ind_fes(mesh, &ind_fec);
    FiniteElementSpace ind_fesv(mesh, &ind_fec, dim);
    GridFunction size(&ind_fes), aspr(&ind_fes), ori(&ind_fes);
@@ -599,6 +634,7 @@ int main(int argc, char *argv[])
          }
          ConstructSizeGF(size);
          tc->SetSerialDiscreteTargetSize(size);
+         tc->SetMinSizeForTargets(size.Min());
          target_c = tc;
          break;
       }
@@ -693,6 +729,7 @@ int main(int argc, char *argv[])
          DiffuseField(aspr, 2);
 
          tc->SetSerialDiscreteTargetSize(size);
+         tc->SetMinSizeForTargets(size.Min());
          tc->SetSerialDiscreteTargetAspectRatio(aspr);
          target_c = tc;
          break;
@@ -740,6 +777,7 @@ int main(int argc, char *argv[])
          ConstantCoefficient size_coeff(0.1*0.1);
          size.ProjectCoefficient(size_coeff);
          tc->SetSerialDiscreteTargetSize(size);
+         tc->SetMinSizeForTargets(size.Min());
 
          FunctionCoefficient ori_coeff(discrete_ori_2d);
          ori.ProjectCoefficient(ori_coeff);
@@ -823,7 +861,7 @@ int main(int argc, char *argv[])
 
    // Limit the node movement.
    // The limiting distances can be given by a general function of space.
-   FiniteElementSpace dist_fespace(mesh, fec); // scalar space
+   FiniteElementSpace dist_fespace(mesh, &fec_h1); // scalar space
    GridFunction dist(&dist_fespace);
    dist = 1.0;
    // The small_phys_size is relevant only with proper normalization.
@@ -867,20 +905,22 @@ int main(int argc, char *argv[])
    // normalization factors for these terms as well.
    if (normalization) { tmop_integ->EnableNormalization(x0); }
 
-   // 12. Setup the final NonlinearForm (which defines the integral of interest,
-   //     its first and second derivatives). Here we can use a combination of
-   //     metrics, i.e., optimize the sum of two integrals, where both are
-   //     scaled by used-defined space-dependent weights. Note that there are no
-   //     command-line options for the weights and the type of the second
-   //     metric; one should update those in the code.
-   NonlinearForm a(fespace);
+   //
+   // Setup the NonlinearForm which defines the integral of interest, its
+   // first and second derivatives.
+   //
+   // Note that the TMOP optimization always operates on H1 spaces. For periodic
+   // meshes, TMOP solves for a continuous periodic displacement.
+   NonlinearForm a(&fes_h1);
    if (pa) { a.SetAssemblyLevel(AssemblyLevel::PARTIAL); }
+   // We can use a combination of metrics, i.e., optimize the sum of two
+   // integrals, where both are scaled by used-defined space-dependent weights.
+   // Note that there are no command-line options for the weights and the type
+   // of the second metric; one should update those in the code.
    ConstantCoefficient *metric_coeff1 = NULL;
    TMOP_QualityMetric *metric2 = NULL;
    TargetConstructor *target_c2 = NULL;
    FunctionCoefficient metric_coeff2(weight_fun);
-
-   // Explicit combination of metrics.
    if (combomet > 0)
    {
       // First metric.
@@ -914,11 +954,8 @@ int main(int argc, char *argv[])
 
       a.AddDomainIntegrator(combo);
    }
-   else
-   {
-      a.AddDomainIntegrator(tmop_integ);
-   }
-
+   else { a.AddDomainIntegrator(tmop_integ); }
+   // The PA setup must be performed after all integrators have been added.
    if (pa) { a.Setup(); }
 
    // Compute the minimum det(J) of the starting mesh.
@@ -949,7 +986,7 @@ int main(int argc, char *argv[])
                   "Untangling is supported only for ideal targets.");
 
       const DenseMatrix &Wideal =
-         Geometries.GetGeomToPerfGeomJac(fespace->GetFE(0)->GetGeomType());
+         Geometries.GetGeomToPerfGeomJac(mesh->GetTypicalElementGeometry());
       min_detJ /= Wideal.Det();
 
       // Slightly below minJ0 to avoid div by 0.
@@ -957,14 +994,15 @@ int main(int argc, char *argv[])
    }
 
    // For HR tests, the energy is normalized by the number of elements.
-   const real_t init_energy = a.GetGridFunctionEnergy(x) /
+   if (periodic) { tmop_integ->SetInitialMeshPos(&x0); }
+   const real_t init_energy = a.GetGridFunctionEnergy(periodic ? dx : x) /
                               (hradaptivity ? mesh->GetNE() : 1);
    real_t init_metric_energy = init_energy;
    if (lim_const > 0.0 || adapt_lim_const > 0.0)
    {
       lim_coeff.constant = 0.0;
       adapt_lim_coeff.constant = 0.0;
-      init_metric_energy = a.GetGridFunctionEnergy(x) /
+      init_metric_energy = a.GetGridFunctionEnergy(periodic ? dx : x) /
                            (hradaptivity ? mesh->GetNE() : 1);
       lim_coeff.constant = lim_const;
       adapt_lim_coeff.constant = adapt_lim_const;
@@ -978,11 +1016,11 @@ int main(int argc, char *argv[])
       vis_tmop_metric_s(mesh_poly_deg, *metric, *target_c, *mesh, title, 0);
    }
 
-   // 13. Fix all boundary nodes, or fix only a given component depending on the
-   //     boundary attributes of the given mesh. Attributes 1/2/3 correspond to
-   //     fixed x/y/z components of the node. Attribute 4 corresponds to an
-   //     entirely fixed node. Other boundary attributes do not affect the node
-   //     movement boundary conditions.
+   // Fix all boundary nodes, or fix only a given component depending on the
+   // boundary attributes of the given mesh.
+   // Attributes 1/2/3 correspond to fixed x/y/z components of the node.
+   // Attribute 4 corresponds to an entirely fixed node.
+   // All other attributes represent unconstrained boundary nodes.
    if (move_bnd == false)
    {
       Array<int> ess_bdr(mesh->bdr_attributes.Max());
@@ -994,7 +1032,7 @@ int main(int argc, char *argv[])
       int n = 0;
       for (int i = 0; i < mesh->GetNBE(); i++)
       {
-         const int nd = fespace->GetBE(i)->GetDof();
+         const int nd = fes_h1.GetBE(i)->GetDof();
          const int attr = mesh->GetBdrElement(i)->GetAttribute();
          MFEM_VERIFY(!(dim == 2 && attr == 3),
                      "Boundary attribute 3 must be used only for 3D meshes. "
@@ -1003,13 +1041,13 @@ int main(int argc, char *argv[])
          if (attr == 1 || attr == 2 || attr == 3) { n += nd; }
          if (attr == 4) { n += nd * dim; }
       }
-      Array<int> ess_vdofs(n);
+      Array<int> vdofs, ess_vdofs(n);
       n = 0;
       for (int i = 0; i < mesh->GetNBE(); i++)
       {
-         const int nd = fespace->GetBE(i)->GetDof();
+         const int nd = fes_h1.GetBE(i)->GetDof();
          const int attr = mesh->GetBdrElement(i)->GetAttribute();
-         fespace->GetBdrElementVDofs(i, vdofs);
+         fes_h1.GetBdrElementVDofs(i, vdofs);
          if (attr == 1) // Fix x components.
          {
             for (int j = 0; j < nd; j++)
@@ -1092,7 +1130,7 @@ int main(int argc, char *argv[])
    // Perform the nonlinear optimization.
    //
    const IntegrationRule &ir =
-      irules->Get(fespace->GetFE(0)->GetGeomType(), quad_order);
+      irules->Get(mesh->GetTypicalElementGeometry(), quad_order);
    TMOPNewtonSolver solver(ir, solver_type);
    // Provide all integration rules in case of a mixed mesh.
    solver.SetIntegrationRules(*irules, quad_order);
@@ -1109,8 +1147,8 @@ int main(int argc, char *argv[])
    }
    // Level of output.
    IterativeSolver::PrintLevel newton_print;
-   if (verbosity_level > 0)
-   { newton_print.Errors().Warnings().Iterations(); }
+   if (verbosity_level > 0) { newton_print.Errors().Warnings().Iterations(); }
+   else                     { newton_print.Errors().Warnings(); }
    solver.SetPrintLevel(newton_print);
    // hr-adaptivity solver.
    // If hr-adaptivity is disabled, r-adaptivity is done once using the
@@ -1124,6 +1162,7 @@ int main(int argc, char *argv[])
                           mesh_poly_deg, h_metric_id,
                           n_hr_iter, n_h_iter);
    hr_solver.AddGridFunctionForUpdate(&x0);
+   hr_solver.AddFESpaceForUpdate(&fes_h1);
    if (adapt_lim_const > 0.)
    {
       hr_solver.AddGridFunctionForUpdate(&adapt_lim_gf0);
@@ -1131,8 +1170,8 @@ int main(int argc, char *argv[])
    }
    hr_solver.Mult();
 
-   // 15. Save the optimized mesh to a file. This output can be viewed later
-   //     using GLVis: "glvis -m optimized.mesh".
+   // Save the optimized mesh to a file. This output can be viewed later
+   // using GLVis: "glvis -m optimized.mesh".
    {
       ofstream mesh_ofs("optimized.mesh");
       mesh_ofs.precision(14);
@@ -1140,14 +1179,21 @@ int main(int argc, char *argv[])
    }
 
    // Report the final energy of the functional.
-   const real_t fin_energy = a.GetGridFunctionEnergy(x) /
+   if (periodic)
+   {
+      GridFunction dx_L2(x); dx_L2 -= x0;
+      // Assumes Gauss-Lobatto and continuity in x and x_0 across faces.
+      dx.ProjectGridFunction(dx_L2);
+   }
+   if (periodic) { tmop_integ->SetInitialMeshPos(&x0); }
+   const real_t fin_energy = a.GetGridFunctionEnergy(periodic ? dx : x) /
                              (hradaptivity ? mesh->GetNE() : 1);
    real_t fin_metric_energy = fin_energy;
    if (lim_const > 0.0 || adapt_lim_const > 0.0)
    {
       lim_coeff.constant = 0.0;
       adapt_lim_coeff.constant = 0.0;
-      fin_metric_energy = a.GetGridFunctionEnergy(x) /
+      fin_metric_energy = a.GetGridFunctionEnergy(periodic ? dx : x) /
                           (hradaptivity ? mesh->GetNE() : 1);
       lim_coeff.constant = lim_const;
       adapt_lim_coeff.constant = adapt_lim_const;
