@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -80,7 +80,6 @@ constexpr real_t NL_DMAX = std::numeric_limits<real_t>::max();
 // Static variables for GLVis
 constexpr int GLVIZ_W = 1024;
 constexpr int GLVIZ_H = 1024;
-constexpr int  visport = 19916;
 constexpr char vishost[] = "localhost";
 
 // Context/Options for the solver
@@ -94,6 +93,8 @@ struct Opt
    int refine = 2;
    int niters = 8;
    int surface = 5;
+   // Socket to send visualization data
+   int visport = 19916;
    bool pa = true;
    bool vis = true;
    bool amr = false;
@@ -156,7 +157,7 @@ public:
    int Solve()
    {
       // Initialize GLVis server if 'visualization' is set
-      if (opt.vis) { opt.vis = glvis.open(vishost, visport) == 0; }
+      if (opt.vis) { opt.vis = glvis.open(vishost, opt.visport) == 0; }
       // Send to GLVis the first mesh
       if (opt.vis) { Visualize(glvis, opt, mesh, GLVIZ_W, GLVIZ_H); }
       // Create and launch the surface solver
@@ -176,7 +177,7 @@ public:
       return 0;
    }
 
-   ~Surface()
+   ~Surface() override
    {
       if (opt.vis) { glvis.close(); }
       delete mesh; delete fec; delete fes;
@@ -457,7 +458,7 @@ public:
       ByNodes(Surface &S, Opt &opt): Solver(S, opt)
       { a.AddDomainIntegrator(new VectorDiffusionIntegrator(one)); }
 
-      bool Step()
+      bool Step() override
       {
          x = *S.fes->GetMesh()->GetNodes();
          bool converge = ParAXeqB();
@@ -495,7 +496,7 @@ public:
       ByVDim(Surface &S, Opt &opt): Solver(S, opt)
       { a.AddDomainIntegrator(new DiffusionIntegrator(one)); }
 
-      bool Step()
+      bool Step() override
       {
          bool cvg[SDIM] {false};
          for (int c=0; c < SDIM; ++c)
@@ -522,7 +523,7 @@ struct Catenoid: public Surface
 {
    Catenoid(Opt &opt): Surface((opt.Tptr = Parametrization, opt)) { }
 
-   void Prefix()
+   void Prefix() override
    {
       SetCurvature(opt.order, false, SDIM, Ordering::byNODES);
       Array<int> v2v(GetNV());
@@ -611,7 +612,7 @@ struct Hold: public Surface
 {
    Hold(Opt &opt): Surface((opt.Tptr = Parametrization, opt)) { }
 
-   void Prefix()
+   void Prefix() override
    {
       SetCurvature(opt.order, false, SDIM, Ordering::byNODES);
       Array<int> v2v(GetNV());
@@ -779,7 +780,7 @@ struct Costa: public Surface
 {
    Costa(Opt &opt): Surface((opt.Tptr = Parametrization, opt), false) { }
 
-   void Prefix()
+   void Prefix() override
    {
       ALPHA[3] = opt.tau;
       const int nx = opt.nx, ny = opt.ny;
@@ -861,7 +862,7 @@ struct Costa: public Surface
       ALPHA[2] = std::fmax(p[2], ALPHA[2]);
    }
 
-   void Snap()
+   void Snap() override
    {
       Vector node(SDIM);
       MFEM_VERIFY(ALPHA[0] > 0.0,"");
@@ -925,7 +926,7 @@ struct FullPeach: public Surface
    FullPeach(Opt &opt):
       Surface((opt.niters = std::min(4, opt.niters), opt), NV, NE, 0) { }
 
-   void Prefix()
+   void Prefix() override
    {
       const real_t quad_v[NV][SDIM] =
       {
@@ -947,9 +948,9 @@ struct FullPeach: public Surface
       SetCurvature(opt.order, false, SDIM, Ordering::byNODES);
    }
 
-   void Snap() { SnapNodesToUnitSphere(); }
+   void Snap() override { SnapNodesToUnitSphere(); }
 
-   void BoundaryConditions()
+   void BoundaryConditions() override
    {
       Vector X(SDIM);
       Array<int> dofs;
@@ -1018,7 +1019,7 @@ struct QuarterPeach: public Surface
       p[2] = 1.0 - gamma;
    }
 
-   void Postfix()
+   void Postfix() override
    {
       for (int i = 0; i < GetNBE(); i++)
       {
@@ -1054,7 +1055,7 @@ struct SlottedSphere: public Surface
 {
    SlottedSphere(Opt &opt): Surface((opt.niters = 4, opt), 64, 40, 0) { }
 
-   void Prefix()
+   void Prefix() override
    {
       constexpr real_t delta = 0.15;
       constexpr int NV1D = 4;
@@ -1146,7 +1147,7 @@ struct SlottedSphere: public Surface
       FinalizeTopology();
    }
 
-   void Snap() { SnapNodesToUnitSphere(); }
+   void Snap() override { SnapNodesToUnitSphere(); }
 };
 
 static int Problem0(Opt &opt)
@@ -1183,17 +1184,17 @@ enum {NORM, AREA};
 static real_t qf(const int order, const int ker, Mesh &m,
                  FiniteElementSpace &fes, GridFunction &u)
 {
-   const Geometry::Type type = m.GetElementBaseGeometry(0);
+   const Geometry::Type type = m.GetTypicalElementGeometry();
    const IntegrationRule &ir(IntRules.Get(type, order));
    const QuadratureInterpolator *qi(fes.GetQuadratureInterpolator(ir));
 
    const int NE(m.GetNE());
-   const int ND(fes.GetFE(0)->GetDof());
+   const int ND(fes.GetTypicalFE()->GetDof());
    const int NQ(ir.GetNPoints());
    const int flags = GeometricFactors::JACOBIANS|GeometricFactors::DETERMINANTS;
    const GeometricFactors *geom = m.GetGeometricFactors(ir, flags);
 
-   const int D1D = fes.GetFE(0)->GetOrder() + 1;
+   const int D1D = fes.GetTypicalFE()->GetOrder() + 1;
    const int Q1D = IntRules.Get(Geometry::SEGMENT, ir.GetOrder()).GetNPoints();
    MFEM_VERIFY(ND == D1D*D1D, "");
    MFEM_VERIFY(NQ == Q1D*Q1D, "");
@@ -1259,7 +1260,7 @@ static int Problem1(Opt &opt)
    FunctionCoefficient u0_fc(u0);
    u.ProjectCoefficient(u0_fc);
    socketstream glvis;
-   if (opt.vis) { opt.vis = glvis.open(vishost, visport) == 0; }
+   if (opt.vis) { opt.vis = glvis.open(vishost, opt.visport) == 0; }
    if (opt.vis) { Surface::Visualize(glvis, opt, &mesh, GLVIZ_W, GLVIZ_H, &u); }
    Vector B, X;
    OperatorPtr A;
@@ -1309,6 +1310,7 @@ int main(int argc, char *argv[])
 
    // Parse command-line options.
    OptionsParser args(argc, argv);
+   args.AddOption(&opt.visport, "-p", "--send-port", "Socket for GLVis.");
    args.AddOption(&opt.pb, "-p", "--problem", "Problem to solve.");
    args.AddOption(&opt.mesh_file, "-m", "--mesh", "Mesh file to use.");
    args.AddOption(&opt.wait, "-w", "--wait", "-no-w", "--no-wait",
@@ -1341,6 +1343,7 @@ int main(int argc, char *argv[])
                   "Enable or disable result output (files in mfem format).");
    args.AddOption(&opt.snapshot, "-ss", "--snapshot", "-no-ss", "--no-snapshot",
                   "Enable or disable GLVis snapshot.");
+   args.AddOption(&opt.visport, "-p", "--send-port", "Socket for GLVis.");
    args.Parse();
    if (!args.Good()) { args.PrintUsage(mfem::out); return 1; }
    MFEM_VERIFY(opt.lambda >= 0.0 && opt.lambda <= 1.0,"");
