@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -143,7 +143,7 @@ public:
    Dist_Level_Set_Coefficient(int type_)
       : Coefficient(), type(type_) { }
 
-   virtual real_t Eval(ElementTransformation &T, const IntegrationPoint &ip)
+   real_t Eval(ElementTransformation &T, const IntegrationPoint &ip) override
    {
       Vector x(3);
       T.Transform(ip, x);
@@ -166,7 +166,7 @@ public:
 
    int GetNLevelSets() { return dls.Size(); }
 
-   virtual real_t Eval(ElementTransformation &T, const IntegrationPoint &ip)
+   real_t Eval(ElementTransformation &T, const IntegrationPoint &ip) override
    {
       MFEM_VERIFY(dls.Size() > 0,
                   "Add at least 1 Dist_level_Set_Coefficient to the Combo.");
@@ -191,8 +191,8 @@ public:
 
    using VectorCoefficient::Eval;
 
-   virtual void Eval(Vector &p, ElementTransformation &T,
-                     const IntegrationPoint &ip)
+   void Eval(Vector &p, ElementTransformation &T,
+             const IntegrationPoint &ip) override
    {
       Vector x;
       T.Transform(ip, x);
