@@ -1,4 +1,4 @@
-// Copyright (c) 2010-2024, Lawrence Livermore National Security, LLC. Produced
+// Copyright (c) 2010-2025, Lawrence Livermore National Security, LLC. Produced
 // at the Lawrence Livermore National Laboratory. All Rights reserved. See files
 // LICENSE and NOTICE for details. LLNL-CODE-806117.
 //
@@ -30,7 +30,7 @@ public:
         is_3D_(is_3D) {}
 
    void ExportToParaView();
-   void SendToGLVis() const;
+   void SendToGLVis(int visport = 19916) const;
 
 private:
    ParMesh *mesh_;
